//=============================================================================
// CASIO Mobile Device CameraLib Sample                                       +
// Copyright (C) 2005 CASIO COMPUTER CO., LTD. All Rights Reserved.           +
//=============================================================================
// dialog.c                                                        +
//-----------------------------------------------------------------------------
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <CameraLib.h>
#include <SystemLib.h>
#include "resource.h"

#define MAX_LOADSTRING 100
#define dim(x)				(sizeof(x) / sizeof(x[0]))
#define SAVE_FOLDER			L"\\My Documents\\CameraLib_Sample"


// Structure
typedef struct tagINTMODELIST{
	int		nMode;
	UINT	wID;
} INTMODELIST;

typedef struct tagDWORDMODELIST{
	DWORD	dwMode;
	UINT	wID;
} DWORDMODELIST;


// Global variable
extern HINSTANCE	hInst;					// Instance
int					nImgSize = CAM_VGA;		// Size
int					nFormat = CAM_BMP;		// File format

// Size
const static INTMODELIST SizeList[] = {
	{CAM_1M,			IDS_SIZE_1M},
	{CAM_XGA,			IDS_SIZE_XGA},
	{CAM_SVGA,			IDS_SIZE_SVGA},
	{CAM_VGA,			IDS_SIZE_VGA},
	{CAM_4PER9VGA,		IDS_SIZE_4PER9VGA},
	{CAM_QVGA,			IDS_SIZE_1PER4VGA},
	{CAM_1PER9VGA,		IDS_SIZE_1PER9VGA},
};

// Diaphragm
const static DWORDMODELIST IrisList[] = {
	{CAM_IRIS_35,		IDS_IRIS_F35},
	{CAM_IRIS_70,		IDS_IRIS_F70},
};

// WiteBalance
const static INTMODELIST LightList[] = {
	{CAM_LIGHT_AUTO,	IDS_LIGHT_AUTO},
	{CAM_OUTDOOR,		IDS_LIGHT_OUTDOOR},
	{CAM_FLUORESCENT,	IDS_LIGHT_FLUORESCENT},
	{CAM_INCANDESCE,	IDS_LIGHT_INCANDESCE},
};


BOOL				InitDialog			(HWND);
BOOL				MakeSaveFaileName	(LPCTSTR, LPTSTR);
void				ShowErrorMessage	(HWND, LPCTSTR, int);


LRESULT CALLBACK DlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	TCHAR szFileName[MAX_PATH];
	int nRet;
	int i;

	switch(message)
	{
	case WM_INITDIALOG:
		// Start processing
		nRet = CAMOpen();
		ShowErrorMessage(hDlg, L"CAMOpen", nRet);

		InitDialog(hDlg);

		// Start Preview
		// IT-600 DT-5200
//		nRet = CAMStartPreview(hDlg, 10, 10, CAM_1PER9VGA);
		// DT-X30 DT-5300 IT-800
		nRet = CAMStartPreview(hDlg, 10, 10, CAM_1PER9VGA | CAM_ROTATE_90);
		ShowErrorMessage(hDlg, L"CAMStartPreview", nRet);
		break;

	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_IMAGESIZE:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// Size
				i = ComboBox_GetCurSel((HWND)lParam);
				nImgSize = SizeList[i].nMode;
			}
			break;

		case IDC_IRIS:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// Diaphragm
				i = ComboBox_GetCurSel((HWND)lParam);
				nRet = CAMSetIris(IrisList[i].dwMode);
			}
			break;

		case IDC_LIGHTMODE:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// WiteBalance
				i = ComboBox_GetCurSel((HWND)lParam);
				nRet = CAMSetLightMode(LightList[i].nMode);
				ShowErrorMessage(hDlg, L"CAMSetLightMode", nRet);
			}
			break;

		case IDC_ILLUMINATION:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// LED
				i = ComboBox_GetCurSel((HWND)lParam);
				nRet = CAMSetIllumination(i);
				ShowErrorMessage(hDlg, L"CAMSetIllumination", nRet);
			}
			break;

		case IDC_SHUTTER:
			CreateDirectory(SAVE_FOLDER, NULL);
			MakeSaveFaileName(SAVE_FOLDER, szFileName);

			// Take a photograph
			nRet = CAMSingleCapture(szFileName, nFormat, nImgSize);
			ShowErrorMessage(hDlg, L"CAMSingleCapture", nRet);
			break;
		}
		break;

	case WM_CLOSE:
		// End processing
		CAMIlluminationOn(FALSE);
		CAMStopPreview();
		CAMClose();
		break;

	case WM_DESTROY:
		break;
	}
	return 0;
}


BOOL InitDialog(HWND hDlg)
{
	TCHAR szTemp[MAX_LOADSTRING];
	DWORD dwMode;
	int nRet;
	int nSel;
	int i;

	//
	for(i = 0 ; i < dim(SizeList); i++)
	{
		LoadString(hInst, SizeList[i].wID, szTemp, MAX_LOADSTRING);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_IMAGESIZE), szTemp);
		if(SizeList[i].nMode == nImgSize)
		{
			nSel = i;
		}
	}
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_IMAGESIZE), nSel);

	// Diaphragm
	nRet = CAMGetIris(&dwMode);

	for(i = 0, nSel = 0 ; i < dim(IrisList); i++)
	{
		LoadString(hInst, IrisList[i].wID, szTemp, MAX_LOADSTRING);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_IRIS), szTemp);
		if(IrisList[i].dwMode == dwMode)
		{
			nSel = i;
		}
	}
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_IRIS), nSel);

	// WiteBalance
	nRet = CAMGetLightMode();
	if(nRet == CAM_NOTOPENED)
	{
		ShowErrorMessage(hDlg, L"CAMGetLightMode", nRet);
	}

	for(i = 0, nSel = 0 ; i < dim(LightList); i++)
	{
		LoadString(hInst, LightList[i].wID, szTemp, MAX_LOADSTRING);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_LIGHTMODE), szTemp);
		if(LightList[i].nMode == nRet)
		{
			nSel = i;
		}
	}
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_LIGHTMODE), nSel);

	// LED
	dwMode = 50;
	nRet = CAMSetIllumination(dwMode);
	ShowErrorMessage(hDlg, L"CAMSetIllumination", nRet);

	for(i = 0 ; i <= 100; i++)
	{
		wsprintf(szTemp, L"%d", i);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_ILLUMINATION), szTemp);
	}
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_ILLUMINATION), dwMode);

	//
	nRet = CAMIlluminationOn(TRUE);
	ShowErrorMessage(hDlg, L"CAMIlluminationOn", nRet);

	return TRUE;
}


// Create file name
BOOL MakeSaveFaileName(LPCTSTR pszFolder, LPTSTR pszFileName)
{
	SYSTEMTIME st;

	if(!pszFolder || !pszFileName)
	{
		return FALSE;
	}

	GetLocalTime(&st);
	wsprintf(pszFileName, L"%s\\%04d%02d%02d_%02d%02d%02d%03d",
			 pszFolder, st.wYear, st.wMonth, st.wDayOfWeek,
			 st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);

	if(nFormat == CAM_BMP)
	{
		wcscat(pszFileName, L".bmp");
	}
	else
	{
		wcscat(pszFileName, L".jpg");
	}
	return TRUE;
}


// Display error message
void ShowErrorMessage(HWND hWnd, LPCTSTR pszCaption, int nStatus)
{
	TCHAR szTitle[MAX_LOADSTRING];
	TCHAR szMsg[MAX_LOADSTRING];

	if(pszCaption && wcslen(pszCaption))
	{
		wcscpy(szTitle, pszCaption);
	}
	else
	{
		LoadString(hInst, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	}

	//Error Message
	switch(nStatus)
	{
	case CAM_SUCCESS:
		return;

	case CAM_NOTOPENED:
		wcscpy(szMsg, L"CAM_NOTOPENED");
		break;

	case CAM_INVALID_PARAM:
		wcscpy(szMsg, L"CAM_INVALID_PARAM");
		break;

	case CAM_INTERNAL_ERROR:
		wcscpy(szMsg, L"CAM_INTERNAL_ERROR");
		break;

	case CAM_OPENERROR:
		wcscpy(szMsg, L"CAM_OPENERROR");
		break;

	case CAM_NOT_ENOUGH_MEMORY:
		wcscpy(szMsg, L"CAM_NOT_ENOUGH_MEMORY");
		break;

	case CAM_SHOW_ERROR:
		wcscpy(szMsg, L"CAM_SHOW_ERROR");
		break;

	case CAM_NOT_STARTED:
		wcscpy(szMsg, L"CAM_NOT_STARTED");
		break;

	case CAM_FILE_ERROR:
		wcscpy(szMsg, L"CAM_FILE_ERROR");
		break;

	case FUNCTION_UNSUPPORT:
		wcscpy(szMsg, L"FUNCTION_UNSUPPORT");
		break;

	default:
		wsprintf(szMsg, L"Unknown Status\n0x%08X", nStatus);
		break;
	}

	// Display MessageBox
	MessageBox(hWnd, szMsg, szTitle, MB_OK | MB_ICONSTOP);
}
