/***************************************************************************
'* <Description>
'* Form1.cs (Casio Computer Co., Ltd.)
'* -------------------------------------------------------------------------
'* <Function outline>
'* Camera Movie Capture Demo program
'* -------------------------------------------------------------------------
'* <Language>
'* C#.NET
'* -------------------------------------------------------------------------
'* <Develop Environment>
'* VS 2005
'* -------------------------------------------------------------------------
'* <Target>
'* DT-5300WM / IT-800 model
'* -------------------------------------------------------------------------
'* Copyright(C)2010 CASIO COMPUTER CO.,LTD. All rights reserved.
'* -------------------------------------------------------------------------
'* <History>
'* Version  Date            Company     Keyword     Comment
'* 1.0.0    2010.03.02      CASIO       000000      Original      
'* 
'***************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using Calib;
using System.Threading;

namespace MovieCS001
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        [DllImport("coredll.dll")]
        public static extern IntPtr GetForegroundWindow();

        int iRet;

        int iImgQuality;
        int[] nImgQuality = new int[] {CameraLibNet.Def.CAM_VIDEO_HIGHEST,
                                    CameraLibNet.Def.CAM_VIDEO_HIGH,
                                    CameraLibNet.Def.CAM_VIDEO_NORMAL,
                                    CameraLibNet.Def.CAM_VIDEO_LOW};
        int iLightList;
        int[] nLightList = new int[] {CameraLibNet.Def.CAM_LIGHT_AUTO,
                                      CameraLibNet.Def.CAM_OUTDOOR,
                                      CameraLibNet.Def.CAM_FLUORESCENT,
                                      CameraLibNet.Def.CAM_INCANDESCE,
                                      16};

        bool bLEDList;
        bool[] nLEDList = new bool[] { false, true };

        private void Form1_GotFocus(object sender, EventArgs e)
        {
            IntPtr hwnd;
            int i;

            iRet = CameraLibNet.Api.CAMOpen();

            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMOpen...OK";
            Application.DoEvents();

            iRet = CameraLibNet.Api.CAMSetIllumination(0);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMSetIllumination...OK";
            Application.DoEvents();

            iRet = CameraLibNet.Api.CAMSetCaptureMode(CameraLibNet.Def.CAM_QVGA, CameraLibNet.Def.CAM_VIDEOCAPTURE_MODE);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMSetCaptureMode...OK";
            Application.DoEvents();

            ComboBox1.SelectedIndex = 2;    // STD
            ComboBox3.SelectedIndex = 0;    // AUTO
            ComboBox5.SelectedIndex = 0;    // LED OFF

            iImgQuality = nImgQuality[ComboBox1.SelectedIndex];
            
            hwnd = GetForegroundWindow();

            iRet = CameraLibNet.Api.CAMStartPreview(hwnd, 120, 10, CameraLibNet.Def.CAM_QVGA);

            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMStartPreview...OK";
            Application.DoEvents();

            for (i = 0; i < 100; i++)
                ComboBox4.Items.Add(i);

            ComboBox4.SelectedIndex = 50;

            iRet = CameraLibNet.Api.CAMIlluminationOn(false);

            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMIlluminationOn...OK";
            Application.DoEvents();

            iRet = CameraLibNet.Api.CAMSetPreviewMode(CameraLibNet.Def.CAM_PREVIEW_FOCUSFRAME, 0);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMSetPreviewMode...OK";
            Application.DoEvents();
        }

        private void DispErrorMessage(int iRet)
        {
            switch (iRet)
            {
                //case CameraLibNet.Def.CAM_SUCCESS:
                //    return;

                case CameraLibNet.Def.CAM_NOTOPENED:
                    StatusBar1.Text = "CAM_NOTOPENED";
                    break;

                case CameraLibNet.Def.CAM_INVALID_PARAM:
                    StatusBar1.Text = "CAM_INVALID_PARAM";
                    break;

                case CameraLibNet.Def.CAM_INTERNAL_ERROR:
                    StatusBar1.Text = "CAM_INTERNAL_ERROR";
                    break;

                case CameraLibNet.Def.CAM_OPENERROR:
                    StatusBar1.Text = "CAM_OPENERROR";
                    break;

                case CameraLibNet.Def.CAM_NOT_ENOUGH_MEMORY:
                    StatusBar1.Text = "CAM_NOT_ENOUGH_MEMORY";
                    break;

                case CameraLibNet.Def.CAM_SHOW_ERROR:
                    StatusBar1.Text = "CAM_SHOW_ERROR";
                    break;

                case CameraLibNet.Def.CAM_NOT_STARTED:
                    StatusBar1.Text = "CAM_NOT_STARTED";
                    break;

                case CameraLibNet.Def.CAM_FILE_ERROR:
                    StatusBar1.Text = "CAM_FILE_ERROR";
                    break;

                case CameraLibNet.Def.FUNCTION_UNSUPPORT:
                    StatusBar1.Text = "FUNCTION_UNSUPPORT";
                    break;

                default:
                    StatusBar1.Text = "Unknown Status";
                    break;
            }
        }

        private void Button2_Click(object sender, EventArgs e)
        {
            string str;
            DateTime dt = DateTime.Now;

            str = "\\My Documents\\Image";
            str = str + dt.ToString("yyMMddHHmmss");
            str = str + ".wmv";

            iRet = CameraLibNet.Api.CAMStartVideoCapture(str, CameraLibNet.Def.CAM_QVGA, 0);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMStartVideoCapture...OK";
            Application.DoEvents();
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            CameraLibNet.Api.CAMClose();
            this.Close();
        }

        private void Button3_Click(object sender, EventArgs e)
        {
            Int32 iRet2 = 0;
            IntPtr hwnd;

            iRet = CameraLibNet.Api.CAMStopVideoCapture(0);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMStopVideoCapture...OK";
            Application.DoEvents();

            iRet = CameraLibNet.Api.CAMWaitForVideoEncode(Timeout.Infinite, ref iRet2, 0);

            hwnd = GetForegroundWindow();

            iRet = CameraLibNet.Api.CAMStartPreview(hwnd, 120, 10, CameraLibNet.Def.CAM_QVGA);

            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMStartPreview...OK";
            Application.DoEvents();

        }

        private void Button4_Click(object sender, EventArgs e)
        {
            Int32 iDummy = 0;
            iRet = CameraLibNet.Api.CAMAutoFocus(CameraLibNet.Def.CAM_AF_NORMAL, ref iDummy);

            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMAutoFocus...OK";
            Application.DoEvents();
        }

        private void ComboBox5_SelectedIndexChanged(object sender, EventArgs e)
        {
            bLEDList = nLEDList[ComboBox5.SelectedIndex];

            iRet = CameraLibNet.Api.CAMIlluminationOn(bLEDList);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMIlluminationOn...OK";
            Application.DoEvents();
        }

        private void ComboBox4_SelectedIndexChanged(object sender, EventArgs e)
        {
            iRet = CameraLibNet.Api.CAMSetIllumination(ComboBox4.SelectedIndex);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMSetIllumination...OK";
            Application.DoEvents();
        }

        private void ComboBox3_SelectedIndexChanged(object sender, EventArgs e)
        {
            iLightList = nLightList[ComboBox3.SelectedIndex];

            iRet = CameraLibNet.Api.CAMSetLightMode(iLightList);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMSetLightMode...OK";
            Application.DoEvents();
        }

        private void ComboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            IntPtr hwnd;
            iRet = CameraLibNet.Api.CAMStopPreview();
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMStopPreview...OK";
            Application.DoEvents();


            iImgQuality = nImgQuality[ComboBox1.SelectedIndex];

            iRet = CameraLibNet.Api.CAMSetVideoCaptureSettings(CameraLibNet.Def.CAM_VIDEOFORMAT_WMV, iImgQuality, CameraLibNet.Def.CAM_VIDEO_AUDIO, 0, 0);
            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMSetVideoCaptureSettings...OK";
            Application.DoEvents();


            hwnd = GetForegroundWindow();

            iRet = CameraLibNet.Api.CAMStartPreview(hwnd, 120, 10, CameraLibNet.Def.CAM_QVGA);

            if (iRet != CameraLibNet.Def.CAM_SUCCESS)
            {
                DispErrorMessage(iRet);
                return;
            }
            StatusBar1.Text = "CAMStartPreview...OK";
            Application.DoEvents();
        }

    }
}