/***************************************************************************
 * <Description>
 * CameraLibSampleWMVC.cpp (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * Camera Capture Demo program
 * -------------------------------------------------------------------------
 * <Language>
 * Visual C++
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * DT-5300 WM / IT-800 WM model
 * -------------------------------------------------------------------------
 * Copyright(C)2009 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2010.05.11      CASIO       000000      Original      
 * 
 ***************************************************************************/
//	000000__	Original
//	__000000	Original

#include <windows.h>
#include <windowsx.h>
#include "resource.h"
#include <aygshell.h>

//	000000__	Original
//  CASIO Libaray
#include "SystemLib.h"
#include "CameraLib.h"
//	__000000	Original

#define MAX_LOADSTRING 100
HINSTANCE				hInst;			// current instance

BOOL DlgMain(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
			LPWSTR lpCmdLine, int nShowCmd)
{
	hInst = hInstance;
	DialogBoxW(hInstance, MAKEINTRESOURCE( IDD_MAIN ), NULL, DlgMain);
	return 0;
}

#define dim(x)				(sizeof(x) / sizeof(x[0]))
#define SAVE_FOLDER			L"\\My Documents\\CameraLib_Sample"

// Structure
typedef struct tagINTMODELIST{
	int		nMode;
	UINT	wID;
} INTMODELIST;

typedef struct tagDWORDMODELIST{
	DWORD	dwMode;
	UINT	wID;
} DWORDMODELIST;

typedef struct tagBOOLMODELIST{
	BOOL	bMode;
	UINT	wID;
} BOOLMODELIST;

// Global variable
extern HINSTANCE	hInst;					// Instance
int					nImgSize = CAM_VGA;		// Size
int					nFormat = CAM_JPEG;		// File format

// Size
const static INTMODELIST SizeList[] = {
	{CAM_UXGA,			IDS_SIZE_UXGA},
	{CAM_QUADVGA,		IDS_SIZE_QUADVGA},
	{CAM_XGA,			IDS_SIZE_XGA},
	{CAM_SVGA,			IDS_SIZE_SVGA},
	{CAM_VGA,			IDS_SIZE_VGA},
	{CAM_QVGA,			IDS_SIZE_QVGA},
};

// FlashList
const static INTMODELIST FlashList[] = {
	{CAM_FLASH_ENABLE,		IDS_FLASH_ENABLE},
	{CAM_FLASH_DISABLE,		IDS_FLASH_DISABLE},
};

// IlluminationList
const static BOOLMODELIST IlluminationList[] = {
	{false,		IDS_ILLUMINATION_OFF},
	{true,		IDS_ILLUMINATION_ON},
};

// WiteBalance
const static INTMODELIST LightList[] = {
	{CAM_LIGHT_AUTO,	IDS_LIGHT_AUTO},
	{CAM_OUTDOOR,		IDS_LIGHT_OUTDOOR},
	{CAM_FLUORESCENT,	IDS_LIGHT_FLUORESCENT},
	{CAM_INCANDESCE,	IDS_LIGHT_INCANDESCE},
	{CAM_DIMLIGHT,		IDS_LIGHT_DIMLIGHT},
};
//
//
BOOL				InitDialog			(HWND);
BOOL				MakeSaveFaileName	(LPCTSTR, LPTSTR);
void				ShowErrorMessage	(HWND, LPCTSTR, int);


BOOL DlgMain(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	TCHAR szFileName[MAX_PATH];
	int nRet;
	int i;
	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			RECT rc;
			#define MENU_HEIGHT 26

            // Create a Done button and size it.  
            SHINITDLGINFO shidi;
            shidi.dwMask = SHIDIM_FLAGS;
			shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_FULLSCREENNOMENUBAR ;
            shidi.hDlg = hDlg;
            SHInitDialog(&shidi);
				
			GetWindowRect(hDlg, &rc);
			MoveWindow(hDlg, rc.left, rc.top, rc.right, rc.bottom+MENU_HEIGHT, TRUE);

			SetForegroundWindow(hDlg);
			SHFullScreen(hDlg, SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON | SHFS_HIDESTARTICON);	


			// Start processing
			nRet = CAMOpen();
			ShowErrorMessage(hDlg, L"CAMOpen", nRet);

			InitDialog(hDlg);

			nRet = CAMStartPreview(hDlg, 130, 10, CAM_QVGA);
			ShowErrorMessage(hDlg, L"CAMStartPreview", nRet);

		}
        return (INT_PTR)TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_EXIT:
			CAMIlluminationOn(FALSE);
			CAMStopPreview();
			CAMClose();

			EndDialog(hDlg, LOWORD(wParam));
			break;
		case IDC_LEDFLASH:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// Flash Enable or Disable
				i = ComboBox_GetCurSel((HWND)lParam);
				CAMSetFlash(FlashList[i].nMode, NULL);
			}
			break;
		case IDC_LEDON:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// Illumination ON or OFF
				i = ComboBox_GetCurSel((HWND)lParam);
				CAMIlluminationOn(IlluminationList[i].bMode);
			}
			break;
		case IDC_LEDSIZE:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// Illumination Size (0 to 100)
				i = ComboBox_GetCurSel((HWND)lParam);
				CAMSetIllumination(i);
			}
			break;
		case IDC_AF:
			CAMAutoFocus(CAM_AF_NORMAL, NULL);
			break;
		case IDC_IMAGESIZE:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// Size
				i = ComboBox_GetCurSel((HWND)lParam);
				nImgSize = SizeList[i].nMode;
			}
			break;
		case IDC_LIGHTMODE:
			if(HIWORD(wParam) == CBN_SELENDOK){
				// Light Mode
				i = ComboBox_GetCurSel((HWND)lParam);
				CAMSetLightMode(LightList[i].nMode);
			}
			break;
		case IDC_SHUTTER:
			CreateDirectory(SAVE_FOLDER, NULL);
			MakeSaveFaileName(SAVE_FOLDER, szFileName);

			// Take a photograph
			nRet = CAMSingleCapture(szFileName, nFormat, nImgSize);
			ShowErrorMessage(hDlg, L"CAMSingleCapture", nRet);
			break;
		}
		break;

	case WM_CLOSE:
				// End processing
		CAMIlluminationOn(FALSE);
		CAMStopPreview();
		CAMClose();

		DestroyWindow(hDlg);
		break;
	}
	return FALSE;
}

// Create file name
BOOL MakeSaveFaileName(LPCTSTR pszFolder, LPTSTR pszFileName)
{
	SYSTEMTIME st;

	if(!pszFolder || !pszFileName)
	{
		return FALSE;
	}

	GetLocalTime(&st);
	wsprintf(pszFileName, L"%s\\%04d%02d%02d_%02d%02d%02d%03d",
			 pszFolder, st.wYear, st.wMonth, st.wDayOfWeek,
			 st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);

	if(nFormat == CAM_BMP)
	{
		wcscat(pszFileName, L".bmp");
	}
	else
	{
		wcscat(pszFileName, L".jpg");
	}
	return TRUE;
}



// error message display
void ShowErrorMessage(HWND hWnd, LPCTSTR pszCaption, DWORD dwStatus)
{
	TCHAR szTitle[MAX_LOADSTRING];
	TCHAR szMsg[MAX_LOADSTRING];
	UINT idError;

	// message box caption
	if(pszCaption && wcslen(pszCaption))
	{
		wcscpy(szTitle, pszCaption);
	}
	else
	{
		LoadString(hInst, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	}

	//error message
	if(dwStatus == SYS_PARAMERR)			idError = IDS_ERR_PARAMETER;
	else if(dwStatus == FUNCTION_UNSUPPORT)	idError = IDS_ERR_UNSUPPORT;
	else									return;

	// message box display
	LoadString(hInst, idError, szMsg, MAX_LOADSTRING);
	MessageBox(hWnd, szMsg, szTitle, MB_OK | MB_ICONSTOP);
}

BOOL InitDialog(HWND hDlg)
{
	TCHAR szTemp[MAX_LOADSTRING];
	DWORD dwMode;
	int nRet;
	int nSel;
	int i;

	//
	for(i = 0 ; i < dim(SizeList); i++)
	{
		LoadString(hInst, SizeList[i].wID, szTemp, MAX_LOADSTRING);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_IMAGESIZE), szTemp);
		if(SizeList[i].nMode == nImgSize)
		{
			nSel = i;
		}
	}
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_IMAGESIZE), nSel);

	// Flash
	nRet = CAMGetFlash(&dwMode, NULL);
	ShowErrorMessage(hDlg, L"CAMGetFlash", nRet);

	for(i = 0, nSel = 0 ; i < dim(FlashList); i++)
	{
		LoadString(hInst, FlashList[i].wID, szTemp, MAX_LOADSTRING);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_LEDFLASH), szTemp);
		if(FlashList[i].nMode == nRet)
		{
			nSel = i;
		}
	}
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_LEDFLASH), nSel);

	// Ilumination
	for(i = 0, nSel = 0 ; i < dim(IlluminationList); i++)
	{
		LoadString(hInst, IlluminationList[i].wID, szTemp, MAX_LOADSTRING);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_LEDON), szTemp);

	}
	nSel = 0; //false / LED OFF
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_LEDON), nSel);

	// WiteBalance
	nRet = CAMGetLightMode();
	if(nRet == CAM_NOTOPENED)
	{
		ShowErrorMessage(hDlg, L"CAMGetLightMode", nRet);
	}

	for(i = 0, nSel = 0 ; i < dim(LightList); i++)
	{
		LoadString(hInst, LightList[i].wID, szTemp, MAX_LOADSTRING);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_LIGHTMODE), szTemp);
		if(LightList[i].nMode == nRet)
		{
			nSel = i;
		}
	}
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_LIGHTMODE), nSel);

	// LED
	dwMode = 50;
	nRet = CAMSetIllumination(dwMode);
	ShowErrorMessage(hDlg, L"CAMSetIllumination", nRet);

	for(i = 0 ; i <= 100; i++)
	{
		wsprintf(szTemp, L"%d", i);
		ComboBox_AddString(GetDlgItem(hDlg, IDC_LEDSIZE), szTemp);
	}
	ComboBox_SetCurSel(GetDlgItem(hDlg, IDC_LEDSIZE), dwMode);

	//
	nRet = CAMIlluminationOn(false);
	ShowErrorMessage(hDlg, L"CAMIlluminationOn", nRet);

	nRet = CAMSetPreviewMode(CAM_PREVIEW_FOCUSFRAME, NULL);

	return TRUE;
}



// Display error message
void ShowErrorMessage(HWND hWnd, LPCTSTR pszCaption, int nStatus)
{
	TCHAR szTitle[MAX_LOADSTRING];
	TCHAR szMsg[MAX_LOADSTRING];

	if(pszCaption && wcslen(pszCaption))
	{
		wcscpy(szTitle, pszCaption);
	}
	else
	{
		LoadString(hInst, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	}

	//Error Message
	switch(nStatus)
	{
	case CAM_SUCCESS:
		return;

	case CAM_NOTOPENED:
		wcscpy(szMsg, L"CAM_NOTOPENED");
		break;

	case CAM_INVALID_PARAM:
		wcscpy(szMsg, L"CAM_INVALID_PARAM");
		break;

	case CAM_INTERNAL_ERROR:
		wcscpy(szMsg, L"CAM_INTERNAL_ERROR");
		break;

	case CAM_OPENERROR:
		wcscpy(szMsg, L"CAM_OPENERROR");
		break;

	case CAM_NOT_ENOUGH_MEMORY:
		wcscpy(szMsg, L"CAM_NOT_ENOUGH_MEMORY");
		break;

	case CAM_SHOW_ERROR:
		wcscpy(szMsg, L"CAM_SHOW_ERROR");
		break;

	case CAM_NOT_STARTED:
		wcscpy(szMsg, L"CAM_NOT_STARTED");
		break;

	case CAM_FILE_ERROR:
		wcscpy(szMsg, L"CAM_FILE_ERROR");
		break;

	case FUNCTION_UNSUPPORT:
		wcscpy(szMsg, L"FUNCTION_UNSUPPORT");
		break;

	default:
		wsprintf(szMsg, L"Unknown Status\n0x%08X", nStatus);
		break;
	}

	// Display MessageBox
	MessageBox(hWnd, szMsg, szTitle, MB_OK | MB_ICONSTOP);
}
