using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Calib;

namespace GestureSampleWMCS
{
    public partial class Form1 : Form
    {
        // Gesture difine
        private const uint WM_GESTURE = 0x0119;
        private const uint GID_BEGIN = 1;
        private const uint GID_END = 2;
        private const uint GID_PAN = 4;
        private const uint GID_TWOFINGERTAP = 6;
        private const uint GID_PRESSANDTAP = 7;
        private const uint GID_SCROLL = 8;
        private const uint GID_HOLD = 9;
        private const uint GID_SELECT = 10;
        private const uint GID_DOUBLESELECT = 11;
        private const uint GID_LAST = 11;

        [DllImport("coredll.dll")]
        private static extern bool GetGestureInfo(IntPtr hGestureInfo, ref GESTUREINFO pGestureInfo);
        private delegate int WndProcDelegate(IntPtr hwnd, uint msg, uint wParam, int lParam);
        private WndProcDelegate wndProc;
        private IntPtr wndProcPtr;
        private IntPtr oldProcPtr;

        [DllImport("coredll.dll")]
        private extern static IntPtr SetWindowLong(IntPtr hwnd, int nIndex, IntPtr dwNewLong);
        private const int GWL_WNDPROC = -4;

        [DllImport("coredll.dll")]
        public extern static int CallWindowProc(IntPtr lpPrevWndFunc, IntPtr hwnd, uint msg, uint wParam, int lParam);

        [StructLayout(LayoutKind.Sequential)]
        private struct GESTUREINFO
        {
            public int cbSize;
            public int dwFlags;
            public int dwID;
            public IntPtr hwndTarget;
            [MarshalAs(UnmanagedType.Struct)]
            public POINTS ptsLocation;
            public int dwInstanceID;
            public int dwSequenceID;
            public UInt64 ullArguments;
            public uint cbExtraArgs;
        }

        [StructLayout(LayoutKind.Sequential)]
        private struct POINTS
        {
            public short x;
            public short y;
        }

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            wndProc = new WndProcDelegate(WndProc);
            wndProcPtr = Marshal.GetFunctionPointerForDelegate(wndProc);
            oldProcPtr = SetWindowLong(this.Handle, GWL_WNDPROC, wndProcPtr);
        }

        protected virtual int WndProc(IntPtr hwnd, uint msg, uint wParam, int lParam)
        {
            Int32 iDistance = 0;
            Int32 iAngle = 0;

            switch (msg)
            {
                case WM_GESTURE:
                    Calib.SystemLibNet.Api.SysConvertGestureMessage(ref msg, ref wParam, ref lParam, ref iDistance, ref iAngle);
                    return OnGesture(hwnd, wParam, lParam, iDistance, iAngle);
            }
            return CallWindowProc(oldProcPtr, hwnd, msg, wParam, lParam); ;
        }

        private Int32 OnGesture(IntPtr hwnd, uint wParam, int lParam, Int32 iDistance, Int32 iAngle)
        {
            GESTUREINFO gi = new GESTUREINFO();
            gi.cbSize = Marshal.SizeOf(typeof(GESTUREINFO));
            bool bRet = GetGestureInfo((IntPtr)lParam, ref gi);
            string strGestureMessage = "";

            if (bRet)
            {
                switch (wParam)
                {
                    case GID_BEGIN:
                    case GID_END:
                        break;

                    case GID_PAN:
                        strGestureMessage = "GID_PAN\nX: " + gi.ptsLocation.x.ToString() + ", Y: " + gi.ptsLocation.y.ToString();
                        labelGesture.Text = strGestureMessage;
                        break;

                    case GID_SCROLL:
                        strGestureMessage = "GID_SCROLL\nX: " + gi.ptsLocation.x.ToString() + ", Y: " + gi.ptsLocation.y.ToString();
                        labelGesture.Text = strGestureMessage;
                        break;

                    case GID_HOLD:
                        strGestureMessage = "GID_HOLD\nX: " + gi.ptsLocation.x.ToString() + ", Y: " + gi.ptsLocation.y.ToString();
                        labelGesture.Text = strGestureMessage;
                        break;

                    case GID_SELECT:
                        strGestureMessage = "GID_SELECT\nX: " + gi.ptsLocation.x.ToString() + ", Y: " + gi.ptsLocation.y.ToString();
                        labelGesture.Text = strGestureMessage;
                        break;

                    case GID_DOUBLESELECT:
                        strGestureMessage = "GID_DOUBLESELECT\nX: " + gi.ptsLocation.x.ToString() + ", Y: " + gi.ptsLocation.y.ToString();
                        labelGesture.Text = strGestureMessage;
                        break;

                    case Calib.SystemLibNet.Def.GID_ROTATE:
                        strGestureMessage = "GID_ROTATE\nAngle: " + iAngle.ToString();
                        labelGesture.Text = strGestureMessage;
                        break;

                    case Calib.SystemLibNet.Def.GID_ZOOM:
                        strGestureMessage = "GID_ZOOM\nDistance: " + iDistance.ToString();
                        labelGesture.Text = strGestureMessage;
                        break;
                }
            }
            return CallWindowProc(oldProcPtr, hwnd, WM_GESTURE, wParam, lParam);
        }

        private void menuExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
    }
}