// GestureSampleWMVC.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "GestureSampleWMVC.h"
#include <windows.h>
#include <commctrl.h>

// Additional Header file
#include "Gesture.h"	// WM_GESTURE message
#include "SystemLib.h"	// MultiTouchGesture convert

#define MAX_LOADSTRING 100

// Additional Define value
#define LABEL_WIDTH		240		// Gesture result display label size
#define LABEL_HEIGHT	125		// Gesture result display label size

// Global Variables:
HINSTANCE			g_hInst;			// current instance
HWND				g_hWndMenuBar;		// menu bar handle

// Additional Global Variable
HWND				g_hWndLabel;		// Gesture result display label

// Forward declarations of functions included in this code module:
ATOM			MyRegisterClass(HINSTANCE, LPTSTR);
BOOL			InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

// Additional event handler
LRESULT WINAPI		OnGesture( HWND, WPARAM, LPARAM, DWORD, DWORD );	// GestureEventHandler

int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPTSTR    lpCmdLine,
                   int       nCmdShow)
{
	MSG msg;

	// Perform application initialization:
	if (!InitInstance(hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	HACCEL hAccelTable;
	hAccelTable = LoadAccelerators(hInstance, MAKEINTRESOURCE(IDC_GESTURESAMPLEWMVC));

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return (int) msg.wParam;
}

//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS wc;

	wc.style         = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc   = WndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;
	wc.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_GESTURESAMPLEWMVC));
	wc.hCursor       = 0;
	wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
	wc.lpszMenuName  = 0;
	wc.lpszClassName = szWindowClass;

	return RegisterClass(&wc);
}

//
//   FUNCTION: InitInstance(HINSTANCE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    HWND hWnd;
    TCHAR szTitle[MAX_LOADSTRING];		// title bar text
    TCHAR szWindowClass[MAX_LOADSTRING];	// main window class name

    g_hInst = hInstance; // Store instance handle in our global variable

    // SHInitExtraControls should be called once during your application's initialization to initialize any
    // of the device specific controls such as CAPEDIT and SIPPREF.
    SHInitExtraControls();

    LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING); 
    LoadString(hInstance, IDC_GESTURESAMPLEWMVC, szWindowClass, MAX_LOADSTRING);

    //If it is already running, then focus on the window, and exit
    hWnd = FindWindow(szWindowClass, szTitle);	
    if (hWnd) 
    {
        // set focus to foremost child window
        // The "| 0x00000001" is used to bring any owned windows to the foreground and
        // activate them.
        SetForegroundWindow((HWND)((ULONG) hWnd | 0x00000001));
        return 0;
    } 

    if (!MyRegisterClass(hInstance, szWindowClass))
    {
    	return FALSE;
    }

    hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

    if (!hWnd)
    {
        return FALSE;
    }

    // When the main window is created using CW_USEDEFAULT the height of the menubar (if one
    // is created is not taken into account). So we resize the window after creating it
    // if a menubar is present
    if (g_hWndMenuBar)
    {
        RECT rc;
        RECT rcMenuBar;

        GetWindowRect(hWnd, &rc);
        GetWindowRect(g_hWndMenuBar, &rcMenuBar);
        rc.bottom -= (rcMenuBar.bottom - rcMenuBar.top);
		
        MoveWindow(hWnd, rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top, FALSE);
    }

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);


    return TRUE;
}

//
//  FUNCTION: WndProc(HWND, UINT, WPARAM, LPARAM)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    int wmId, wmEvent;
    PAINTSTRUCT ps;
    HDC hdc;

	// Gesture variable
	DWORD dwDistance = 0;	// Distance between 2 points
	DWORD dwAngle = 0;		// Angle rotation between 2 points

	// Gesture result display label variable
	int iLabelLeft, iLabelTop;

    static SHACTIVATEINFO s_sai;
	
    switch (message) 
    {
		// Gesture message !!
		case WM_GESTURE:
			// SystemLib GestureMessage convert
			SysConvertGestureMessage( &message, &wParam, &lParam, &dwDistance, &dwAngle );
			return OnGesture( hWnd, wParam, lParam, dwDistance, dwAngle );

        case WM_COMMAND:
            wmId    = LOWORD(wParam); 
            wmEvent = HIWORD(wParam); 
            // Parse the menu selections:
            switch (wmId)
            {
                case IDM_HELP_ABOUT:
                    DialogBox(g_hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, About);
                    break;
                case IDM_OK:
                    SendMessage (hWnd, WM_CLOSE, 0, 0);				
                    break;
                default:
                    return DefWindowProc(hWnd, message, wParam, lParam);
            }
            break;
        case WM_CREATE:
            SHMENUBARINFO mbi;

            memset(&mbi, 0, sizeof(SHMENUBARINFO));
            mbi.cbSize     = sizeof(SHMENUBARINFO);
            mbi.hwndParent = hWnd;
            mbi.nToolBarId = IDR_MENU;
            mbi.hInstRes   = g_hInst;

            if (!SHCreateMenuBar(&mbi)) 
            {
                g_hWndMenuBar = NULL;
            }
            else
            {
                g_hWndMenuBar = mbi.hwndMB;
            }

            // Initialize the shell activate info structure
            memset(&s_sai, 0, sizeof (s_sai));
            s_sai.cbSize = sizeof (s_sai);

			// Gesture result display label
			RECT rect;
			GetClientRect(hWnd, &rect);
			iLabelLeft = (rect.right >> 1) - (LABEL_WIDTH >> 1);
			iLabelTop = (rect.bottom >> 1) - (LABEL_HEIGHT >> 1);
			g_hWndLabel = CreateWindowEx(0, TEXT("STATIC"),	0, WS_CHILD | WS_VISIBLE | SS_CENTER, iLabelLeft,
								 iLabelTop, LABEL_WIDTH, LABEL_HEIGHT, hWnd, NULL, g_hInst, NULL);

            break;
        case WM_PAINT:
            hdc = BeginPaint(hWnd, &ps);
            
            // TODO: Add any drawing code here...
            
            EndPaint(hWnd, &ps);
            break;
        case WM_DESTROY:
            CommandBar_Destroy(g_hWndMenuBar);
            PostQuitMessage(0);
            break;

        case WM_ACTIVATE:
            // Notify shell of our activate message
            SHHandleWMActivate(hWnd, wParam, lParam, &s_sai, FALSE);
            break;
        case WM_SETTINGCHANGE:
            SHHandleWMSettingChange(hWnd, wParam, lParam, &s_sai);
            break;

        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
    }
    return 0;
}

// Message handler for about box.
INT_PTR CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            {
                // Create a Done button and size it.  
                SHINITDLGINFO shidi;
                shidi.dwMask = SHIDIM_FLAGS;
                shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN | SHIDIF_EMPTYMENU;
                shidi.hDlg = hDlg;
                SHInitDialog(&shidi);
            }
            return (INT_PTR)TRUE;

        case WM_COMMAND:
            if (LOWORD(wParam) == IDOK)
            {
                EndDialog(hDlg, LOWORD(wParam));
                return TRUE;
            }
            break;

        case WM_CLOSE:
            EndDialog(hDlg, message);
            return TRUE;

    }
    return (INT_PTR)FALSE;
}

// GestureEventHandler
LRESULT WINAPI OnGesture( HWND hWnd, WPARAM wParam, LPARAM lParam, DWORD dwDistance, DWORD dwAngle )
{
	// Gesture information store structure variable
	GESTUREINFO gi;
	ZeroMemory( &gi, sizeof( GESTUREINFO ) );
	gi.cbSize = sizeof( GESTUREINFO );

	// Gesture information get
	if( TKGetGestureInfo( (HGESTUREINFO)lParam, &gi ) )
	{
		TCHAR strGestureInfo[256];

		// Some Gesture routine
		// Output Gesture information into label control
		switch( wParam )
		{
			case GID_BEGIN:
			case GID_END:
				return DefWindowProc( hWnd, WM_GESTURE, wParam, lParam );

			// PAN operation
			case GID_PAN:
				wsprintf( strGestureInfo, TEXT("GID_PAN\nX: %d, Y: %d"), gi.ptsLocation.x, gi.ptsLocation.y );
				SetWindowText( g_hWndLabel, strGestureInfo );
				break;

			// Scroll operation
			case GID_SCROLL:
				wsprintf( strGestureInfo, TEXT("GID_SCROLL\nX: %d, Y: %d"), gi.ptsLocation.x, gi.ptsLocation.y );
				SetWindowText( g_hWndLabel, strGestureInfo );
				break;

			// Tap operation
			case GID_SELECT:
				wsprintf( strGestureInfo, TEXT("GID_SELECT\nX: %d, Y: %d"), gi.ptsLocation.x, gi.ptsLocation.y );
				SetWindowText( g_hWndLabel, strGestureInfo );
				break;

			// Double tap operation
			case GID_DOUBLESELECT:
				wsprintf( strGestureInfo, TEXT("GID_DOUBLESELECT\nX: %d, Y: %d"), gi.ptsLocation.x, gi.ptsLocation.y );
				SetWindowText( g_hWndLabel, strGestureInfo );
				break;

			// Hold operation
			case GID_HOLD:
				wsprintf( strGestureInfo, TEXT("GID_HOLD\nX: %d, Y: %d"), gi.ptsLocation.x, gi.ptsLocation.y );
				SetWindowText( g_hWndLabel, strGestureInfo );
				break;

			// Zoom operation
			case GID_ZOOM:
				wsprintf( strGestureInfo, TEXT("GID_ZOOM\nDistance: %d"), dwDistance );
				SetWindowText( g_hWndLabel, strGestureInfo );
				break;

			// Rotate operation
			case GID_ROTATE:
				wsprintf( strGestureInfo, TEXT("GID_ROTATE\nAngle: %d"), dwAngle );
				SetWindowText( g_hWndLabel, strGestureInfo );
				break;

			default:
				break;
		}
	}

	return ERROR_SUCCESS;
}
