/***************************************************************************
'* <Description>
'* MCReadLibSampleCECS.cs (Casio Computer Co., Ltd.)
'* -------------------------------------------------------------------------
'* <Function outline>
'* MCRead Demo program
'* -------------------------------------------------------------------------
'* <Language>
'* C#.NET
'* -------------------------------------------------------------------------
'* <Develop Environment>
'* VS 2005
'* -------------------------------------------------------------------------
'* <Target>
'* IT-9000 WM model
'* -------------------------------------------------------------------------
'* Copyright(C)2012 CASIO COMPUTER CO.,LTD. All rights reserved.
'* -------------------------------------------------------------------------
'* <History>
'* Version  Date            Company     Keyword     Comment
'* 1.0.0    2012.05.24      CASIO       000000      Original      
'* 
'***************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using Calib;

namespace MCReadLibSampleWMCS
{
    public partial class MCReadLibSampleWMCS : Form
    {
        public MCReadLibSampleWMCS()
        {
            InitializeComponent();
        }

        [DllImport("coredll.dll")]
        public static extern IntPtr GetForegroundWindow();

        public const Int32 MCR_TRUE = 1;
        public const Int32 MCR_FALSE = 0;

        public const Int32 WM_USER = 0x0400;
        public const Int32 WM_MSR_READING = (WM_USER + 0x528);

        private void MCReadLibSampleCECS_GotFocus(object sender, EventArgs e)
        {
            IntPtr hwnd;
            Int32 iRet;

            hwnd = GetForegroundWindow();

            iRet = MCReadLibNet.Api.MCROpen(hwnd);

            if (iRet != (Int32)(MCReadLibNet.Def.STATUS.MCR_DATA_SUCCESS))
            {
                DispErrorMessage(iRet);
                return;
            }
            statusBar1.Text = "MCROpen...OK";

            MCReadLibNet.Api.MCRSetEventNotification(true);

            Application.DoEvents();
        }

        private void MCReadLibSampleCECS_Tick(object sender, EventArgs e)
        {
            Int32 iRet;
            byte[] pDataJIS2 = new byte[128];
            byte[] pDataISO1 = new byte[128];
            byte[] pDataISO2 = new byte[128];
            byte[] pDataISO3 = new byte[128];
            Int32 nStatJIS2;
            Int32 nStatISO1;
            Int32 nStatISO2;
            Int32 nStatISO3;
            string str;
            int wint;
            int i;

            iRet = MCReadLibNet.Api.MCRGetStatus();
            if (iRet == (Int32)(MCReadLibNet.Def.STATUS.MCR_DATA_SUCCESS))
            {
                statusBar1.Text = "MCRGetStatus...OK";

                iRet = MCReadLibNet.Api.MCRRead(out pDataJIS2, out pDataISO1, out pDataISO2, out pDataISO3,
                                                out nStatJIS2, out nStatISO1, out nStatISO2, out nStatISO3);

                if (nStatISO1 == (Int32)(MCReadLibNet.Def.STATUS.MCR_DATA_SUCCESS))
                {
                    statusBar1.Text = "MCRRead ISO1...OK";
                    str = "";
                    for (i = 0; i < 76; i++)
                    {
                        wint = (int)pDataISO1[i];
                        str += wint.ToString("X2");
                        str += " ";
                    }
                    TextTrack1.Text = str;
                }
                else
                {
                    TextTrack1.Text = "No Read";
                }
                if (nStatISO2 == (Int32)(MCReadLibNet.Def.STATUS.MCR_DATA_SUCCESS))
                {
                    statusBar1.Text = "MCRRead ISO2...OK";
                    str = "";
                    for (i = 0; i < 37; i++)
                    {
                        wint = (int)pDataISO2[i];
                        str += wint.ToString("X2");
                        str += " ";
                    }
                    TextTrack2.Text = str;

                }
                else
                {
                    TextTrack2.Text = "No Read";
                }
                if (nStatISO3 == (Int32)(MCReadLibNet.Def.STATUS.MCR_DATA_SUCCESS))
                {
                    statusBar1.Text = "MCRRead ISO3...OK";
                    str = "";
                    for (i = 0; i < 104; i++)
                    {
                        wint = (int)pDataISO3[i];
                        str += wint.ToString("X2");
                        str += " ";
                    }
                    TextTrack3.Text = str;
                }
                else
                {
                    TextTrack3.Text = "No Read";
                }
            }
        }


        private void DispErrorMessage(int iRet)
        {
            switch (iRet)
            {
                case (Int32)MCReadLibNet.Def.STATUS.MCR_BUFFER_FULL_ERR:
                    statusBar1.Text = "BUFFER FULL ERR";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_DATASHORT_ERR:
                    statusBar1.Text = "DATASHORT ERR";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_PACKET_LRC_ERR:
                    statusBar1.Text = "PACKET_LRC ERR";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_PARITY_ERR:
                    statusBar1.Text = "PARITY ERR";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_TIMEOUT_ERR:
                    statusBar1.Text = "TIMEOUT ERR";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_CARDSPEED_ERR1:
                    statusBar1.Text = "CARDSPEED ERR1";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_CARDSPEED_ERR2:
                    statusBar1.Text = "CARDSPEED ERR2";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_PULSE_ERR1:
                    statusBar1.Text = "PULSE ERR1";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_PULSE_ERR2:
                    statusBar1.Text = "PULSE ERR2";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_DATA_ERR1:
                    statusBar1.Text = "DATA ERR1";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_DATA_ERR2:
                    statusBar1.Text = "DATA ERR2";
                    break;
                case (Int32)MCReadLibNet.Def.STATUS.MCR_DATA_ERROR:
                    statusBar1.Text = "DATA ERROR";
                    break;
                default:
                    statusBar1.Text = "Unknown Status";
                    break;
            }
        }
    
    }
}