/***************************************************************************
 * <Description>
 * NFCHFTagMain.cs (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * NFC HFTag IC Card reading Demo program
 * -------------------------------------------------------------------------
 * <Language>
 * C#.NET
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * IT-9000 CE model
 * -------------------------------------------------------------------------
 * Copyright(C)2012 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2012.05.24      CASIO       000000      Original      
 * 
 ***************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.VisualBasic;

using Calib;

namespace NFCHFTagCE_CS
{
    public partial class NFCHFTagMain : Form
    {
        public NFCHFTagMain()
        {
            InitializeComponent();
        }

        int iRet;
        byte[] byDSFID = new byte[8];
        byte[] pUID = new byte[8];
        byte[] pData = new byte[16];
        int dwActualSize;
        string strCommand;

        private void button1_Click(object sender, EventArgs e)
        {
            int i;

            button1.Enabled = false;

            label6.Text = "";
            label7.Text = "";
            textBox2.Text = "";
            textBox3.Text = "";
            Application.DoEvents();

            //NFCHFTag POLLING
            textBox3.Text = "NFCHFTagPolling...start." + ControlChars.NewLine + textBox3.Text;
            iRet = NFCHFTagLibNet.Api.NFCHFTagPolling(10000, IntPtr.Zero, 0x00, 0, 0);

            if (iRet != NFCHFTagLibNet.Def.NFC_OK)
            {
                strCommand = "NFCHFTagPolling";
                DispErrorMessage(iRet, strCommand);
                return;
            }
            textBox3.Text = "NFCHFTagPolling...OK." + ControlChars.NewLine + textBox3.Text;

            //NFCHFTag GET CARD RESPONSE
            textBox3.Text = "NFCHFTagGetCardResponse...start." + ControlChars.NewLine + textBox3.Text;
            iRet = NFCHFTagLibNet.Api.NFCHFTagGetCardResponse(byDSFID, pUID, 0);
            if (iRet != NFCHFTagLibNet.Def.NFC_OK)
            {
                strCommand = "NFCHFTagGetCardResponse";
                DispErrorMessage(iRet, strCommand);
                return;
            }
            textBox3.Text = "NFCHFTagGetCardResponse...OK." + ControlChars.NewLine + textBox3.Text;

            //OUTPUT DATA(DSFID,UID) 
            for (i = 0; i < 1; i++)
            {
                label6.Text = label6.Text + byDSFID[i].ToString("X2") + " ";
            }
            for (i = 0; i < 8; i++)
            {
                label7.Text = label7.Text + pUID[i].ToString("X2") + " ";
            }

            if (comboBox1.SelectedIndex == 1)
            {
                textBox3.Text = "NFCHFTagRead...start." + ControlChars.NewLine + textBox3.Text;
                iRet = NFCHFTagLibNet.Api.NFCHFTagRead(Convert.ToInt32(textBox1.Text.ToString(), 16), pData, ref dwActualSize, 0, 0);
                if (iRet != NFCHFTagLibNet.Def.NFC_OK)
                {
                    strCommand = "NFCHFTagRead";
                    DispErrorMessage(iRet, strCommand);
                    return;
                }
                textBox3.Text = "NFCHFTagRead...OK." + ControlChars.NewLine + textBox3.Text;
                for (i = 0; i < dwActualSize; i++)
                {
                    textBox2.Text = textBox2.Text + pData[i].ToString("X2");
                }

            }
            else if (comboBox1.SelectedIndex == 2)
            {
                textBox3.Text = "NFCHFTagWrite...start." + ControlChars.NewLine + textBox3.Text;

                // Append your NFCHFTagWrite code.

                textBox3.Text = "NFCHFTagWrite...OK." + ControlChars.NewLine + textBox3.Text;
            }

            // NFCHFTag RADIO OFF
            textBox3.Text = "NFCHFTagRadioOff...start." + ControlChars.NewLine + textBox3.Text;
            iRet = NFCHFTagLibNet.Api.NFCHFTagRadioOff();
            if (iRet != NFCHFTagLibNet.Def.NFC_OK)
            {
                strCommand = "NFCHFTagRadioOff";
                DispErrorMessage(iRet, strCommand);
                return;
            }
            textBox3.Text = "NFCHFTagRadioOff...OK." + ControlChars.NewLine + textBox3.Text;

            button1.Enabled = true;
            textBox3.Text = "NFCHFTag scanning." + ControlChars.NewLine + textBox3.Text;
            SystemLibNet.Api.SysSetBuzzerVolume(SystemLibNet.Def.B_SCANEND, SystemLibNet.Def.BUZZERVOLUME_MAX);
            SystemLibNet.Api.SysPlayBuzzer(SystemLibNet.Def.B_SCANEND, SystemLibNet.Def.B_USERDEF, SystemLibNet.Def.B_USERDEF);

        }

        private void DispErrorMessage(int iRet, string strCommand)
        {
            switch (iRet)
            {
                case NFCHFTagLibNet.Def.NFC_PON:
                    textBox3.Text = strCommand + " : NFC already opend." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_NOT_DEVICE:
                    textBox3.Text = strCommand + " : NFC driver error." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_ERROR_INVALID_ACCESS:
                    textBox3.Text = strCommand + " : Device exclusive error." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_ERROR_MODULE:
                    textBox3.Text = strCommand + " : NFC module not responce error." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_POF:
                    textBox3.Text = strCommand + " : NFC not open error." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_PRM:
                    textBox3.Text = strCommand + " : NFC parameter error." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_ERROR_TIMEOUT:
                    textBox3.Text = strCommand + " : Timeout error." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_ERROR_CALLBACK:
                    textBox3.Text = strCommand + " : Call back function error." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_ERROR_STOP:
                    textBox3.Text = strCommand + " : Stop error by stop function." + ControlChars.NewLine + textBox3.Text;
                    break;
                case NFCHFTagLibNet.Def.NFC_NOT_ACTIVATION:
                    textBox3.Text = strCommand + " : Card don't start error." + ControlChars.NewLine + textBox3.Text;
                    break;
                default:
                    textBox3.Text = strCommand + " : Other error." + ControlChars.NewLine + textBox3.Text;
                    break;
            }
            SystemLibNet.Api.SysSetBuzzerVolume(SystemLibNet.Def.B_WARNING, SystemLibNet.Def.BUZZERVOLUME_MAX);
            SystemLibNet.Api.SysPlayBuzzer(SystemLibNet.Def.B_WARNING, SystemLibNet.Def.B_USERDEF, SystemLibNet.Def.B_USERDEF);

            //NFCHFTagLibNet.Api.NFCHFTagClose();
            button1.Enabled = true;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            Common.ShowTaskbar();
            this.Close();
        }

        private void NFCHFTagMain_Load(object sender, EventArgs e)
        {
            Common.HideTaskbar();
            comboBox1.SelectedIndex = 0;

            textBox3.Text = "NFCHFTagOpen...start." + ControlChars.NewLine + textBox3.Text;

            // NFC OPEN
            iRet = NFCHFTagLibNet.Api.NFCHFTagOpen(IntPtr.Zero);

            if (iRet != NFCHFTagLibNet.Def.NFC_OK)
            {
                strCommand = "NFCHFTagOpen";
                DispErrorMessage(iRet, strCommand);
                return;
            }
            textBox3.Text = "NFCHFTagOpen...OK." + ControlChars.NewLine + textBox3.Text;
        }

        private void NFCHFTagMain_Closed(object sender, EventArgs e)
        {
            // NFC CLOSE
            NFCHFTagLibNet.Api.NFCHFTagClose();
        }
    }
}
