/***************************************************************************
 * <Description>
 * NFCHFTagMain.cpp (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * NFC HFTag IC Card reading Demo program
 * -------------------------------------------------------------------------
 * <Language>
 * Visual C++
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * IT-9000 CE
 * -------------------------------------------------------------------------
 * Copyright(C)2012 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2012.05.22      CASIO       000000      Original      
 * 
 ***************************************************************************/
//	000000__	Original
//	__000000	Original

#include <windows.h>
#include "resource.h"

//	000000__	Original
//  CASIO Libaray
#include "NFCHFTagLib.h"
#include "NFCLib.h"
#include "SystemLib.h"
//	__000000	Original


BOOL DlgNFC(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL NFC_Scan(HWND hDlg);
BOOL DispErrorMessage(HWND hDlg, int iRet, TCHAR* szCommand);

int		iRet;

BYTE	byDSFID;
BYTE	pUid[8];
BYTE	pData[32];
DWORD	dwActualSize;

// Block Address
TCHAR	szBlockAddress[10];
int		iBlockAddressSize;
DWORD	dwBlockAddress;

// For display status information

TCHAR	szMessage[500];
TCHAR	szMessage2[500];

// For display ISO15693 card data information 
TCHAR	szBuffer[261];


int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
			LPWSTR lpCmdLine, int nShowCmd)
{
	DialogBoxW(hInstance, MAKEINTRESOURCE( IDD_NFC ), NULL, DlgNFC);
	return 0;
}

BOOL DlgNFC(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	//HWND hTask;
	switch (uMsg)
	{
	case WM_INITDIALOG:
		// Task bar hide
		ShowWindow(FindWindow(_T("HHTaskBar"), _T("")), SW_HIDE);

		SendMessage(GetDlgItem(hDlg,IDC_COMMAND), CB_ADDSTRING, 0, (LPARAM)_T("READ"));
		SendMessage(GetDlgItem(hDlg,IDC_COMMAND), CB_ADDSTRING, 0, (LPARAM)_T("WRITE"));

		SetDlgItemText(hDlg, IDC_BLOCKADDRESS, _T("0"));

		// NFCHFTag OPEN
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagOpen...start.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);

		iRet = NFCHFTagOpen(NULL);

		if (iRet != NFC_OK)
		{
			DispErrorMessage(hDlg, iRet, _T("NFCHFTagOpen"));
			return (INT_PTR)TRUE;
		}

		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagOpen...OK.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);

		return (INT_PTR)TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_EXIT:
			// Task bar show
			ShowWindow(FindWindow(_T("HHTaskBar"), _T("")), SW_SHOW);

			EndDialog(hDlg, LOWORD(wParam));
			break;

		case IDC_NFC_SCAN:
			NFC_Scan(hDlg);
			break;
		}
		break;
	case WM_CLOSE:
		ShowWindow(FindWindow(_T("HHTaskBar"), _T("")), SW_SHOW);

		EndDialog(hDlg, LOWORD(wParam));
		break;
	case WM_DESTROY:
		// NFCHFTag CLOSE
		NFCHFTagClose();
		break;
	}
	return FALSE;
}

BOOL NFC_Scan(HWND hDlg)
{
	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),false);

	SetDlgItemText(hDlg, IDC_STATUS,  _T(""));
	SetDlgItemText(hDlg, IDC_DSFID,  _T(""));
	SetDlgItemText(hDlg, IDC_UID,  _T(""));
	SetDlgItemText(hDlg, IDC_DATA, _T(""));

	StringCchCopy( szMessage, sizeof(szMessage), _T(""));
	StringCchCopy( szMessage2, sizeof(szMessage2), _T(""));

	UpdateWindow(hDlg);


	StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
	StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagPolling...start.\r\n"));
	StringCchCat( szMessage, sizeof(szMessage), szMessage2);
	SetDlgItemText(hDlg, IDC_STATUS, szMessage);

	// NFCHFTag POLLING
	iRet = NFCHFTagPolling(10000, NULL, 0x00, 0, 0);

	if (iRet != NFC_OK)
	{
		DispErrorMessage(hDlg, iRet, _T("NFCHFTagPolling"));
		return FALSE;
	}
	StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
	StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagPolling...OK.\r\n"));
	StringCchCat( szMessage, sizeof(szMessage), szMessage2);
	SetDlgItemText(hDlg, IDC_STATUS, szMessage);


	byDSFID = 0;
	ZeroMemory(pUid, sizeof(pUid));

	StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
	StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagGetCardResponse...start.\r\n"));
	StringCchCat( szMessage, sizeof(szMessage), szMessage2);
	SetDlgItemText(hDlg, IDC_STATUS, szMessage);


	// NFCHFTag GET CARD RESPONSE
	iRet = NFCHFTagGetCardResponse(&byDSFID, pUid, 0);

	if (iRet != NFC_OK)
	{
		DispErrorMessage(hDlg, iRet, _T("NFCHFTagGetCardResponse"));
		return FALSE;
    }

	StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
	StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagGetCardResponse...OK.\r\n"));
	StringCchCat( szMessage, sizeof(szMessage), szMessage2);
	SetDlgItemText(hDlg, IDC_STATUS, szMessage);


	ZeroMemory(szBuffer, sizeof(szBuffer));

	wsprintf(szBuffer, _T("%02X "), byDSFID);
	SetDlgItemText(hDlg, IDC_DSFID, szBuffer );

	ZeroMemory(szBuffer, sizeof(szBuffer));

	for (int i=0 ; i<8 ; i++)
	{
		wsprintf(&szBuffer[i*3], _T("%02X "), pUid[i]);
	}
	SetDlgItemText(hDlg, IDC_UID, szBuffer );


	// NFCHFTagRead

	if (SendMessage(GetDlgItem(hDlg, IDC_COMMAND), CB_GETCURSEL,0 ,0 ) == 0)
	{
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagRead...start.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);

		ZeroMemory(szBlockAddress, sizeof(szBlockAddress));
		iBlockAddressSize = sizeof(szBlockAddress);

		GetDlgItemText(hDlg, IDC_BLOCKADDRESS, szBlockAddress, iBlockAddressSize);

		dwBlockAddress = (DWORD)::_ttoi(szBlockAddress);

		dwActualSize = 0;

		iRet = NFCHFTagRead(dwBlockAddress, pData, &dwActualSize, 0, 0);

		if (iRet != NFC_OK)
		{
			DispErrorMessage(hDlg, iRet, _T("NFCHFTagRead"));
			return FALSE;
		}

		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagRead...OK.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);

		ZeroMemory(szBuffer, sizeof(szBuffer));

		for (int i=0 ; i<(int)dwActualSize ; i++)
		{
			wsprintf(&szBuffer[i*2], _T("%02X"), pData[i]);
		}
		SetDlgItemText(hDlg, IDC_DATA, szBuffer );
	}
	// NFCHFTagWrite
	else if (SendMessage(GetDlgItem(hDlg,IDC_COMMAND), CB_GETCURSEL,0 ,0 ) == 1)  // Case of WRITE
	{
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagWrite...start.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);


		// Append your NFCHFTagWrite code.


		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagWrite...OK.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
	}


	// NFCHFTag RADIO OFF
	iRet = NFCHFTagRadioOff();

	if (iRet != NFC_OK)
	{
		DispErrorMessage(hDlg, iRet, _T("NFCHFTagRadioOff"));
		return FALSE;
	}

	StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
	StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTagRadioOff...OK.\r\n"));
	StringCchCat( szMessage, sizeof(szMessage), szMessage2);
	SetDlgItemText(hDlg, IDC_STATUS, szMessage);


	SysSetBuzzerVolume(B_SCANEND, BUZZERVOLUME_MAX);
	SysPlayBuzzer(B_SCANEND, B_USERDEF, B_USERDEF);

	StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
	StringCchCopy( szMessage, sizeof(szMessage), _T("NFCHFTag scanning...OK.\r\n"));
	StringCchCat( szMessage, sizeof(szMessage), szMessage2);
	SetDlgItemText(hDlg, IDC_STATUS, szMessage);

	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);

	return TRUE;
}

BOOL DispErrorMessage(HWND hDlg, int iRet, TCHAR* szCommand)
{
	switch (iRet)
	{
	case NFC_PON:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : NFC already opened.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_NOT_DEVICE:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : NFC driver error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_ERROR_INVALID_ACCESS:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : Device exclusive error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_ERROR_MODULE:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : NFC module not responce error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_POF:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : NFC not open error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_PRM:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : NFC parameter error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_ERROR_TIMEOUT:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : Timeout error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_ERROR_CALLBACK:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : Call back function error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_ERROR_STOP:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : Stop error by stop function.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	case NFC_NOT_ACTIVATION:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : Card don't start error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
	    break;
	default:
		StringCchCopy( szMessage2, sizeof(szMessage2), szMessage);
		StringCchCopy( szMessage, sizeof(szMessage), szCommand);
		StringCchCat( szMessage, sizeof(szMessage), _T(" : Other error.\r\n"));
		StringCchCat( szMessage, sizeof(szMessage), szMessage2);
		SetDlgItemText(hDlg, IDC_STATUS, szMessage);
		break;
	}
	SysSetBuzzerVolume(B_WARNING, BUZZERVOLUME_MAX);
	SysPlayBuzzer(B_WARNING, B_USERDEF, B_USERDEF);

	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
	return TRUE;
}