/***************************************************************************
 * <Description>
 * NFCMain.cpp (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * NFC IC Card reading Demo program
 * -------------------------------------------------------------------------
 * <Language>
 * Visual C++
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * DT-5300 WM / IT-9000 WM
 * -------------------------------------------------------------------------
 * Copyright(C)2009 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2009.10.14      CASIO       000000      Original      
 * 1.0.1    2012.05.24      CASIO       000001      Modify        
 * 
 ***************************************************************************/
//	000000__	Original
//	__000000	Original

#include <windows.h>
#include "resource.h"
#include <aygshell.h>

//	000000__	Original
//  CASIO Libaray
#include "NFCLib.h"
#include "SystemLib.h"
//	__000000	Original

BOOL DlgNFC(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL NFC_Scan(HWND hDlg);
BOOL DispErrorMessage(HWND hDlg, int iRet);

//	000001__	Modify
// In case of DT-5300 WM, append following definitions.
//#define DRAW_IC_CARD_IMAGE
//#define HFTAG_UNSUPPORTED
//	__000001	Modify

int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
			LPWSTR lpCmdLine, int nShowCmd)
{
	DialogBoxW(hInstance, MAKEINTRESOURCE( IDD_NFC ), NULL, DlgNFC);
	return 0;
}

BOOL DlgNFC(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	int iRet;
	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			RECT rc;
			#define MENU_HEIGHT 26

            // Create a Done button and size it.  
            SHINITDLGINFO shidi;
            shidi.dwMask = SHIDIM_FLAGS;
			shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_FULLSCREENNOMENUBAR ;
            shidi.hDlg = hDlg;
            SHInitDialog(&shidi);
				
			GetWindowRect(hDlg, &rc);
			MoveWindow(hDlg, rc.left, rc.top, rc.right, rc.bottom+MENU_HEIGHT, TRUE);

			SetForegroundWindow(hDlg);
			SHFullScreen(hDlg, SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON | SHFS_HIDESTARTICON);	
		}
		ShowWindow(GetDlgItem( hDlg, IDC_IC_CARD), SW_HIDE);
		
		//	000001__	Modify
		// NFC OPEN
		iRet = NFCOpen(NULL);

		SetDlgItemText(hDlg, IDC_STATUS, _T("NFCOpen...start."));

		if (iRet != NFC_OK)
		{
			DispErrorMessage(hDlg, iRet);
			return FALSE;
		}
		//	__000001	Modify

        return (INT_PTR)TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_EXIT:
			EndDialog(hDlg, LOWORD(wParam));
			break;

		case IDC_NFC_SCAN:
			NFC_Scan(hDlg);
			break;
		}
		break;
	case WM_CLOSE:
		//	000001__	Modify
		EndDialog(hDlg, LOWORD(wParam));
		//	__000001	Modify
		break;
	case WM_DESTROY:
		//	000001__	Modify
		// NFC CLOSE
		NFCClose();
		//	__000001	Modify
		break;
	}
	return FALSE;
}


BOOL NFC_Scan(HWND hDlg)
{
	TCHAR szBuffer[261];
	BYTE pTargetData[261];
	DWORD pActBaudRate;
	DWORD iLen;
	int iRet;

	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),false);
#if defined(DRAW_IC_CARD_IMAGE)
	ShowWindow(GetDlgItem( hDlg, IDC_IC_CARD), SW_SHOW);
#endif

	SetDlgItemText(hDlg, IDC_STATUS,  _T(""));
	SetDlgItemText(hDlg, IDC_CARD_TYPE,  _T(""));
	SetDlgItemText(hDlg, IDC_LENGTH,  _T(""));
	SetDlgItemText(hDlg, IDC_ID,  _T(""));
	SetDlgItemText(hDlg, IDC_RESPONSE,  _T(""));
			
	UpdateWindow(hDlg);

	SetDlgItemText(hDlg, IDC_STATUS, _T("NFCPollingCard...start."));

	// NFC POLLING
	iRet = NFCPollingCard(NFC_BAUD_TYPEA | 
						  NFC_BAUD_TYPEB | 
						  NFC_BAUD_FELICA1 | 
						  NFC_BAUD_FELICA2
	//	000001__	Modify
#if !defined(HFTAG_UNSUPPORTED)
						  | NFC_BAUD_HFTAG
#endif
	//	__000001	Modify
						  ,
						  10000 , 
						  NULL ,
						  &pActBaudRate,0);

	if (iRet != NFC_OK)
	{
		DispErrorMessage(hDlg, iRet);
		return FALSE;
	}

	SetDlgItemText(hDlg, IDC_STATUS, _T("NFCPollingCard...OK."));

	if (pActBaudRate == NFC_BAUD_TYPEA)
	{
		SetDlgItemText(hDlg, IDC_CARD_TYPE,  _T("Type A"));
	}
	else if (pActBaudRate == NFC_BAUD_TYPEB)
		SetDlgItemText(hDlg, IDC_CARD_TYPE,  _T("Type B"));
	else if (pActBaudRate == NFC_BAUD_FELICA1)
		SetDlgItemText(hDlg, IDC_CARD_TYPE,  _T("FeliCa"));
	else if (pActBaudRate == NFC_BAUD_FELICA2)
		SetDlgItemText(hDlg, IDC_CARD_TYPE,  _T("FeliCa"));
	//	000001__	Modify
#if !defined(HFTAG_UNSUPPORTED)
	else if (pActBaudRate == NFC_BAUD_HFTAG)
		SetDlgItemText(hDlg, IDC_CARD_TYPE,  _T("ISO15693"));
#endif
	//	__000001	Modify

	//	000001__	Modify
	ZeroMemory(pTargetData, sizeof(pTargetData));
	//	__000001	Modify

	SetDlgItemText(hDlg, IDC_STATUS, _T("NFCGetCardResponse...start."));

	// NFC GET CARD RESPONSE
	iRet = NFCGetCardResponse(pTargetData, &iLen, 0);

	if (iRet != NFC_OK)
	{
		DispErrorMessage(hDlg, iRet);
		return FALSE;
    }

	SetDlgItemText(hDlg, IDC_STATUS, _T("NFCGetCardResponse...OK."));

	//	000001__	Modify
	ZeroMemory(szBuffer, sizeof(szBuffer));
	//	__000001	Modify

	wsprintf(szBuffer, _T("%2d"), iLen);
	SetDlgItemText(hDlg, IDC_LENGTH, szBuffer );

	//	000001__	Modify
	ZeroMemory(szBuffer, sizeof(szBuffer));
	//	__000001	Modify

	for (int i=0 ; i<(int)iLen ; i++)
	{
		wsprintf(&szBuffer[i*3], _T("%02X "), pTargetData[i]);
	}
	SetDlgItemText(hDlg, IDC_RESPONSE, szBuffer);

	switch (pActBaudRate)
	{
	case NFC_BAUD_TYPEA:
		int UidLen;
		UidLen = (int)pTargetData[3];
		//	000001__	Modify
		ZeroMemory(szBuffer, sizeof(szBuffer));
		//	__000001	Modify

		for (int i=0 ; i<UidLen ; i++)
		{
			wsprintf(&szBuffer[i*3], _T("%02X "), pTargetData[4+i]);
		}
		SetDlgItemText(hDlg, IDC_ID, szBuffer);


		// Append your NFCExchangeData code.


		break;

	case NFC_BAUD_TYPEB:
		//	000001__	Modify
		ZeroMemory(szBuffer, sizeof(szBuffer));
		for (int i=0 ; i<4 ; i++)
		//	__000001	Modify
		{
			wsprintf(&szBuffer[i*3], _T("%02X "), pTargetData[1+i]);
		}
		SetDlgItemText(hDlg, IDC_ID, szBuffer);


		// Append your NFCExchangeData code.


		break;

	case NFC_BAUD_FELICA1:
	case NFC_BAUD_FELICA2:
		//	000001__	Modify
		ZeroMemory(szBuffer, sizeof(szBuffer));
		for (int i=0 ; i<8 ; i++)
		//	__000001	Modify
		{
			wsprintf(&szBuffer[i*3], _T("%02X "), pTargetData[i]);
		}
		SetDlgItemText(hDlg, IDC_ID, szBuffer);


		// Append your NFCExchangeData code.


		break;
	//	000001__	Modify
#if !defined(HFTAG_UNSUPPORTED)
	case NFC_BAUD_HFTAG:
		ZeroMemory(szBuffer, sizeof(szBuffer));
		for (int i=0 ; i<8 ; i++)
		{
			wsprintf(&szBuffer[i*3], _T("%02X "), pTargetData[1+i]);
		}
		SetDlgItemText(hDlg, IDC_ID, szBuffer);


		// Append your NFCExchangeData code.


		break;
#endif
	//	__000001	Modify
	default:
		break;
	}

	SetDlgItemText(hDlg, IDC_STATUS,  _T("NFCRadioOff...start."));

	// NFC RADIO OFF
	iRet = NFCRadioOff();

	if (iRet != NFC_OK)
	{
		DispErrorMessage(hDlg, iRet);
		return FALSE;
	}

	SetDlgItemText(hDlg, IDC_STATUS,  _T("NFCRadioOff...OK."));

	SysSetBuzzerVolume(B_SCANEND, BUZZERVOLUME_MAX);
	SysPlayBuzzer(B_SCANEND, B_USERDEF, B_USERDEF);

	SetDlgItemText(hDlg, IDC_STATUS,  _T("NFC scanning...OK."));

	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
#if defined(DRAW_IC_CARD_IMAGE)
	ShowWindow(GetDlgItem( hDlg, IDC_IC_CARD), SW_HIDE);
#endif
	return TRUE;
}

BOOL DispErrorMessage(HWND hDlg, int iRet)
{
	switch (iRet)
	{
	case NFC_PON:
		SetDlgItemText(hDlg, IDC_STATUS, _T("Error : NFC already opened."));
		break;
	case NFC_NOT_DEVICE:
		SetDlgItemText(hDlg, IDC_STATUS, _T("Error : NFC driver error."));
		break;
	case NFC_ERROR_INVALID_ACCESS:
		SetDlgItemText(hDlg, IDC_STATUS, _T("Error : Device exclusive error."));
		break;
	case NFC_ERROR_MODULE:
		SetDlgItemText(hDlg, IDC_STATUS, _T("Error : NFC module not responce error."));
		break;
	case NFC_POF:
		SetDlgItemText(hDlg, IDC_STATUS,  _T("Error : NFC not open error."));
		break;
	case NFC_PRM:
		SetDlgItemText(hDlg, IDC_STATUS,  _T("Error : NFC parameter error."));
		break;
	case NFC_ERROR_TIMEOUT:
		SetDlgItemText(hDlg, IDC_STATUS,  _T("Error : Timeout error."));
		break;
	case NFC_ERROR_CALLBACK:
		SetDlgItemText(hDlg, IDC_STATUS,  _T("Error : Call back function error."));
		break;
	case NFC_ERROR_STOP:
		SetDlgItemText(hDlg, IDC_STATUS,  _T("Error : Stop error by stop function."));
		break;
	case NFC_NOT_ACTIVATION:
		SetDlgItemText(hDlg, IDC_STATUS,  _T("Error : Card don't start error."));
	    break;
	default:
		SetDlgItemText(hDlg, IDC_STATUS,  _T("Error : Other error."));
		break;
	}
	SysSetBuzzerVolume(B_WARNING, BUZZERVOLUME_MAX);
	SysPlayBuzzer(B_WARNING, B_USERDEF, B_USERDEF);

	NFCClose();

	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
	ShowWindow(GetDlgItem( hDlg, IDC_IC_CARD), SW_HIDE);
	return TRUE;
}