using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using CalibCs;

namespace PrinGDILibSampleCECS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        
        [DllImport("Coredll.dll")]
        extern static bool LineTo(Int32 hDC, Int32 nXEnd, Int32 nYEnd);
        [DllImport("Coredll.dll")]
        extern static bool PeekMessage(ref MSG lpMsg, Int32 hwnd, Int32 wMsgFilterMin,
            Int32 wMsgFilterMax, int wRemoveMsg);

        [StructLayout(LayoutKind.Sequential)]
        public struct POINTAPI
        {
            public int x;
            public int y;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct MSG
        {
            public IntPtr hWnd;
            public int msg;
            public IntPtr wParam;
            public IntPtr lParam;
            public int time;
            public POINTAPI pt;
        }

        Int32 hPrinter = 0;
        Int32 hDC = 0;

        private void button1_Click(object sender, EventArgs e)
        {
            bool bRet = false;

            // CpOpenPrinter : Opens the specified printer and fetches the handle.
            // Printer name : BuiltIn(the built-in printer) Output port name :
            // BuiltIn(the built-in printer)
            bRet = CalibCs.Cp780LibCS.Api.CpOpenPrinter("BuiltIn", "BuiltIn", ref hPrinter);
            if (bRet == false)
            {
                MessageBox.Show("CpOpenPrinter Error");
                return;
            }

            CalibCs.Cp780LibCS.CPDEVMODE cInMode = new CalibCs.Cp780LibCS.CPDEVMODE();
            CalibCs.Cp780LibCS.CPDEVMODE cOutMode = new CalibCs.Cp780LibCS.CPDEVMODE();

            // CpDocumentProperties : Fetches the printer data.
            // Fetches the size required to store the printer data.
            Int32 iSize = CalibCs.Cp780LibCS.Api.CpDocumentProperties((int)this.Handle, hPrinter, cOutMode, cInMode, CalibCs.Cp780LibCS.Def.CPDM_OUT_BUFFER);
            if (iSize < 0)
            {
                MessageBox.Show("CpDocumentProperties Error");
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                return;
            }

            // Changes the size (length) of drawing.
            // Changes the drawing size (length) to 200(20[mm]).
            cOutMode.dmPaperLength = 200;

            // CpCreateDC : Fetches the device context handle for drawing.
            hDC = CalibCs.Cp780LibCS.Api.CpCreateDC(hPrinter, cOutMode);
            if (hDC == 0)
            {
                MessageBox.Show("CpCreateDC Error");
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                return;
            }

            CalibCs.Cp780LibCS.CPDOCINFO cDocInfo = new Cp780LibCS.CPDOCINFO();
            cDocInfo.cbSize = 136;

            // CpSartDoc : Sets up for starting printing job.
            Int32 iRet = CalibCs.Cp780LibCS.Api.CpStartDoc(hPrinter, hDC, cDocInfo);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpStartDoc Error");
                return;
            }

            // CpStartPage : Sets up for starting the page.
            iRet = CalibCs.Cp780LibCS.Api.CpStartPage(hPrinter, hDC);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpStartPage Error");
                return;
            }

            // Draws a triangle on the device context handle
            LineTo( hDC, 100, 100);
            LineTo( hDC, 100, 0);
            LineTo( hDC, 0, 0);

            // CpEndPage : Sets up for closing the page
            iRet = CalibCs.Cp780LibCS.Api.CpEndPage(hPrinter, hDC);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpEndPage Error");
                return;
            }

            // CpResetDC : Changes the paper size
            cOutMode.dmPaperLength = 100;
            
            // Changes the drawing size (length) from 200 to 100(10[mm]).
            Int32 hDCOld = hDC;
            hDC = CalibCs.Cp780LibCS.Api.CpResetDC(hPrinter, hDC, cOutMode);
            if (hDC == 0)
            {
                MessageBox.Show("CpResetDC Error");
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDCOld);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDCOld);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                return;
            }

            // CpStartPage : Sets up for starting the page
            iRet = CalibCs.Cp780LibCS.Api.CpStartPage(hPrinter, hDC);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpStartPage Error");
                return;
            }

            // CpUnicodeOut : Sends the character string
            iRet = CalibCs.Cp780LibCS.Api.CpUnicodeOut(hPrinter, hDC, "UnicodeOut test", 15);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndPage(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpUnicodeOut Error");
                return;
            }

            // string szESC_E = "\x1b""E";
            Byte[] escdata = new Byte[] { 0x1b, 0x45, 0x00};
            Byte[] dmydata = new Byte[] { 0x00, 0x00, 0x00};
                        
            // CpExtEscape : Sends the command of [ESC]E (discharges non-print data)
            iRet = CalibCs.Cp780LibCS.Api.CpExtEscape(hPrinter, hDC, CalibCs.Cp780LibCS.Def.CPPASSTHROGH, 2, ref escdata[0], 0, ref dmydata[0]);
            
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndPage(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpExtEscape Error");
                return;
            }

            // CpEndPage : Sets up for closing the page
            iRet = CalibCs.Cp780LibCS.Api.CpEndPage(hPrinter, hDC);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpEndPage Error");
                return;
            }

            // CpEndDoc : Sets up for closing the document
            // Printing job starts when executing CpEndDoc function
            iRet = CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpEndDoc Error");
                return;
            }

            // CpDeleteDC : Deletes the device context
            bRet = CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
            if (bRet == false)
            {
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpDeleteDC Error");
                return;
            }

            // CpClosePrinter : Closes the specified printer
            bRet = CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
            if (bRet == false)
            {
                MessageBox.Show("CpClosePrinter Error");
                return;
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            bool bRet = false;
            Int32 iRet;

            // CpOpenPrinter : Opens the specified printer and fetches the handle.
            // Printer name : BuiltIn(the built-in printer) Output port name :
            // BuiltIn(the built-in printer)
            bRet = CalibCs.Cp780LibCS.Api.CpOpenPrinter("BuiltIn", "BuiltIn", ref hPrinter);
            if (bRet == false)
            {
                MessageBox.Show("CpOpenPrinter Error");
                return;
            }

            // CpSetBarcodeType:Specifies the method used to read the barcode on the back
            // bartype = 1;					// Back-side barcode type 1
            // bartype = 2;					// Back-side barcode type 2
            Int32 bartype = 2;
            iRet = CalibCs.Cp780LibCS.Api.CpSetBarcodeType(hPrinter, bartype);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                MessageBox.Show("CpOpenPrinter Error");
                return;
            }

            CalibCs.Cp780LibCS.CPDEVMODE cInMode = new CalibCs.Cp780LibCS.CPDEVMODE();
            CalibCs.Cp780LibCS.CPDEVMODE cOutMode = new CalibCs.Cp780LibCS.CPDEVMODE();

            // CpDocumentProperties : Fetches the printer data
            // Fetches the size required to store the printer data
            Int32 iSize = CalibCs.Cp780LibCS.Api.CpDocumentProperties((int)this.Handle, hPrinter, cOutMode, cInMode, CalibCs.Cp780LibCS.Def.CPDM_OUT_BUFFER);
            if (iSize < 0)
            {
                MessageBox.Show("CpDocumentProperties Error");
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                return;
            }

            // Changing the drawing size (length)
            // If reading the barcode on the back, set the drawing size (length) to at least 420 (42 [mm])
            cOutMode.dmPaperLength = 420;
            cOutMode.bMarkerDetection = CalibCs.Cp780LibCS.Def.MARKER_END;

            // CpCreateDC : Fetches the device context handle for drawing
            hDC = CalibCs.Cp780LibCS.Api.CpCreateDC(hPrinter, cOutMode);
            if (hDC == 0)
            {
                MessageBox.Show("CpCreateDC Error");
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                return;
            }

            CalibCs.Cp780LibCS.CPDOCINFO cDocInfo = new Cp780LibCS.CPDOCINFO();
            cDocInfo.cbSize = 136;

            // CpSartDoc : Sets up for starting printing job
            iRet = CalibCs.Cp780LibCS.Api.CpStartDoc(hPrinter, hDC, cDocInfo);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpStartDoc Error");
                return;
            }

            // CpStartPage : Sets up for starting the page
            iRet = CalibCs.Cp780LibCS.Api.CpStartPage(hPrinter, hDC);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpStartPage Error");
                return;
            }

            // Draws a triangle on the device context handle
            LineTo(hDC, 100, 100);
            LineTo(hDC, 100, 0);
            LineTo(hDC, 0, 0);

            // CpEndPage : Sets up for closing the page
            iRet = CalibCs.Cp780LibCS.Api.CpEndPage(hPrinter, hDC);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpEndPage Error");
                return;
            }

            // CpEndDoc : Sets up for closing the document
            // Printing job starts when executing CpEndDoc function
            iRet = CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
            if (iRet == CalibCs.Cp780LibCS.Def.CP_ERROR)
            {
                CalibCs.Cp780LibCS.Api.CpEndDoc(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpEndDoc Error");
                return;
            }

            // CpDeleteDC : Deletes the device context
            bRet = CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
            if (bRet == false)
            {
                CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
                CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
                MessageBox.Show("CpDeleteDC Error");
                return;
            }

            // Waits for printing to end, then reads in the results of the back-side barcode scan
            // After receiving the CP_WM_PRINTDONE printing completion message,
            // the system reads in the results of the back-side barcode scan.
            // Carry out the following processes from within the window.
            while (true)
            {
                MSG msg = new MSG();
                if (PeekMessage(ref msg, 0,
                    CalibCs.Cp780LibCS.Def.CP_WM_PRINTDONE,
                    CalibCs.Cp780LibCS.Def.CP_WM_PRINTDONE, 0x0001))
                {
                    Int32 len;
                    string szbarcode = "                                ";
                    len = 0;

                    if (CalibCs.Cp780LibCS.Api.CpDecodeBarcode(hPrinter, ref len, out szbarcode) > 0)
                    {
                        MessageBox.Show(szbarcode);
                    }
                    else
                    {
                        MessageBox.Show("CpDecodeBarcode Error");
                    }
                    break;
                }
                System.Threading.Thread.Sleep(100);
            }

            // CpClosePrinter : Closes the specified printer
            bRet = CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
            if (bRet == false)
            {
                MessageBox.Show("CpClosePrinter Error");
                return;
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            CalibCs.Cp780LibCS.Api.CpDeleteDC(hPrinter, hDC);
            CalibCs.Cp780LibCS.Api.CpClosePrinter(hPrinter);
            Application.Exit();
        }
    }
}