/******************************************************************************
*  prnCommon.cpp                                                              *
*                                                                             *
*  Project : Printer Demo Application for DT-9700/IT-3000                     *
*  Date    : 2004-03-05                                                       *
*  Version : 1.0.0                                                            *
*                                                                             *
*  Copyright (C) 2004, CASIO COMPUTER CO.,Ltd. All rights reserved.           *
*                                                                             *
******************************************************************************/
#include    "PrinterLibSampleVC.h"

//-------------------------------------------------------------
//		GraphicViewProc
//-------------------------------------------------------------
LRESULT CALLBACK WINAPI	GraphicViewProc(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam)
{
	PAINTSTRUCT ps; 
	SCROLLINFO si; 

	static HDC hdcScreen;
	static HDC hdcScreenCompat;
	static HBITMAP hbmpCompat;
	static BITMAP bmp;
	static HBITMAP hOldBitmap;
 
	static int yMinScroll;
	static int yCurrentScroll;
	static int yMaxScroll;
	static int xMinScroll;
	static int xCurrentScroll;
	static int xMaxScroll;
	static int resourceId;

	switch (message) 
	{ 
		case WM_CREATE:
			{
			int** pTmp=(int**)lParam;
			resourceId=(int)*(int*)*pTmp;
			si.cbSize = sizeof(SCROLLINFO);
			si.fMask  = SIF_RANGE;
			si.nMin   = 0;
			si.nMax   = 0;
			si.nPage  = 1;
			SetScrollInfo(hWnd, SB_VERT, &si, TRUE);
			SetScrollInfo(hWnd, SB_HORZ, &si, TRUE);
	
			BITMAP bmp; 
			HBITMAP hBitmap = LoadBitmap(g_hInst, MAKEINTRESOURCE(resourceId));
			GetObject(hBitmap, sizeof(BITMAP), &bmp);
	
			RECT rc;
			GetClientRect(hWnd, &rc); 
			int nWidth=rc.right-rc.left; 
			int nHeight=rc.bottom-rc.top; 

			xMaxScroll = max(bmp.bmWidth - nWidth, 0);
			xMinScroll     = 0;
			xCurrentScroll = 0;
	
			si.fMask  = SIF_RANGE | SIF_PAGE | SIF_POS; 
			si.nMin   = xMinScroll; 
			si.nMax   = bmp.bmWidth; 
			si.nPage  = nWidth; 
			si.nPos   = xCurrentScroll; 
			SetScrollInfo(hWnd, SB_HORZ, &si, TRUE); 

			yMaxScroll = max(bmp.bmHeight - nHeight, 0); 
			yMinScroll     = 0;
			yCurrentScroll = 0; 

			si.fMask  = SIF_RANGE | SIF_PAGE | SIF_POS; 
			si.nMin   = yMinScroll; 
			si.nMax   = bmp.bmHeight; 
			si.nPage  = nHeight;
			si.nPos   = yCurrentScroll; 
			SetScrollInfo(hWnd, SB_VERT, &si, TRUE); 

			DeleteObject(hBitmap);
			}
			break;

		case WM_PAINT: 
			{ 
			PRECT prect;
			BeginPaint(hWnd, &ps);
			prect = &ps.rcPaint; 

			hdcScreen = GetDC(hWnd);
			hdcScreenCompat = CreateCompatibleDC(hdcScreen);
			hbmpCompat = LoadBitmap(g_hInst, MAKEINTRESOURCE(resourceId));
			GetObject(hbmpCompat, sizeof(BITMAP), &bmp);
			hOldBitmap = (HBITMAP)SelectObject(hdcScreenCompat,hbmpCompat);

			BitBlt(ps.hdc, 
				prect->left, prect->top, 
				(prect->right - prect->left), 
				(prect->bottom - prect->top), 
				hdcScreenCompat, 
				prect->left + xCurrentScroll, 
				prect->top + yCurrentScroll, 
				SRCCOPY); 

			SelectObject(hdcScreenCompat, hOldBitmap);
			DeleteDC(hdcScreenCompat);
			DeleteObject(hbmpCompat);

			EndPaint(hWnd, &ps); 
			} 
			break; 

		case WM_VSCROLL: 
		{ 
			int xDelta = 0; 
			int yDelta;
			int yNewPos;

			switch (LOWORD(wParam)) 
			{ 
				case SB_PAGEUP: 
					yNewPos = yCurrentScroll - 50; 
					break; 

				case SB_PAGEDOWN: 
					yNewPos = yCurrentScroll + 50; 
					break; 

				case SB_LINEUP: 
					yNewPos = yCurrentScroll - 5; 
					break; 

				case SB_LINEDOWN: 
					yNewPos = yCurrentScroll + 5; 
					break; 

				case SB_THUMBPOSITION: 
					yNewPos = HIWORD(wParam); 
					break; 

				default: 
					yNewPos = yCurrentScroll; 
			} 

			yNewPos = max(0, yNewPos); 
			yNewPos = min(yMaxScroll, yNewPos); 

			if (yNewPos == yCurrentScroll) 
				break; 

			yDelta = yNewPos - yCurrentScroll; 
			yCurrentScroll = yNewPos; 

			ScrollWindowEx(hWnd, -xDelta, -yDelta, (CONST RECT *) NULL, 
				(CONST RECT *) NULL, (HRGN) NULL, (LPRECT) NULL, 
				SW_INVALIDATE); 

			UpdateWindow(hWnd);

			si.cbSize = sizeof(si); 
			si.fMask  = SIF_POS; 
			si.nPos   = yCurrentScroll;

			SetScrollInfo(hWnd, SB_VERT, &si, TRUE); 
 
			} 
			break; 	

		case WM_HSCROLL: 
			{ 
			int xDelta;
			int xNewPos;
			int yDelta = 0; 
 
			switch (LOWORD(wParam)) 
			{ 
				case SB_PAGEUP: 
					xNewPos = xCurrentScroll - 50; 
					break; 

				case SB_PAGEDOWN: 
					xNewPos = xCurrentScroll + 50; 
					break; 

				case SB_LINEUP: 
					xNewPos = xCurrentScroll - 5; 
					break; 

				case SB_LINEDOWN: 
					xNewPos = xCurrentScroll + 5; 
					break; 

				case SB_THUMBPOSITION: 
					xNewPos = HIWORD(wParam); 
					break; 
 
				default: 
					xNewPos = xCurrentScroll; 
			} 
 
			xNewPos = max(0, xNewPos); 
			xNewPos = min(xMaxScroll, xNewPos); 
 
			if (xNewPos == xCurrentScroll) 
				break; 
 
			xDelta = xNewPos - xCurrentScroll; 
			xCurrentScroll = xNewPos; 
 
			ScrollWindowEx(hWnd, -xDelta, -yDelta, (CONST RECT *) NULL, 
				(CONST RECT *) NULL, (HRGN) NULL, (LPRECT) NULL, 
				SW_INVALIDATE); 
			UpdateWindow(hWnd); 
 
			si.cbSize = sizeof(si); 
			si.fMask  = SIF_POS; 
			si.nPos   = xCurrentScroll; 
			SetScrollInfo(hWnd, SB_HORZ, &si, TRUE); 
			} 
			break; 

		case WM_CLOSE:
			DestroyWindow(hWnd); 
			break;

		case WM_DESTROY:
			SelectObject(hdcScreenCompat, hOldBitmap);
			DeleteDC(hdcScreenCompat);
			DeleteObject(hbmpCompat);

			DestroyWindow(hWnd);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}		
	return FALSE;
}

//-------------------------------------------------------------
//		GetImageData
//-------------------------------------------------------------
void GetImageData(IMAGE_DATA* pImage,WORD id) 
{
	HDC hDC;
	BITMAP  bmp;
	BITMAPINFO *pbmi;
	void *pbits=NULL;

	pbmi = (BITMAPINFO*)new BYTE[sizeof(BITMAPINFO)+sizeof(RGBQUAD)*2];
	ZeroMemory(&pbmi->bmiHeader, sizeof(pbmi->bmiHeader));

	HBITMAP hSourceBitmap = LoadBitmap(g_hInst, MAKEINTRESOURCE(id));
	GetObject( hSourceBitmap, sizeof(BITMAP), &bmp);

	pbmi->bmiHeader.biSize   = sizeof(BITMAPINFOHEADER);
	pbmi->bmiHeader.biWidth       = bmp.bmWidth;
	pbmi->bmiHeader.biHeight = -bmp.bmHeight;
	pbmi->bmiHeader.biBitCount    = bmp.bmBitsPixel;
	pbmi->bmiHeader.biCompression = BI_RGB;
	pbmi->bmiHeader.biPlanes      = bmp.bmPlanes;
	pbmi->bmiHeader.biSizeImage =
			(((bmp.bmWidth
				* bmp.bmBitsPixel+31)>>3) 
					& 0xfffffffc)
						* bmp.bmHeight;

	pImage->dwWidth     = bmp.bmWidth;
	pImage->dwHeight    = bmp.bmHeight;
	pImage->dwBitsPixel = bmp.bmBitsPixel;
	pImage->dwPlanes    = bmp.bmPlanes;
	pImage->dwImageSize = pbmi->bmiHeader.biSizeImage;

	pbmi->bmiColors[0].rgbRed	= 0;
	pbmi->bmiColors[0].rgbGreen = 0;
	pbmi->bmiColors[0].rgbBlue	= 0;
	pbmi->bmiColors[1].rgbRed	= 255;
	pbmi->bmiColors[1].rgbGreen = 255;
	pbmi->bmiColors[1].rgbBlue	= 255;


	hDC = GetDC(NULL);
	HBITMAP hTargetBitmap = CreateDIBSection(hDC, pbmi, DIB_RGB_COLORS, &pbits, NULL, 0);
	ReleaseDC(NULL, hDC);

	HDC memDc    = CreateCompatibleDC(NULL);
	HDC targetDc = CreateCompatibleDC(NULL);

	HBITMAP hOldBitmap1 = (HBITMAP)SelectObject(memDc, hSourceBitmap);
	HBITMAP hOldBitmap2 = (HBITMAP)SelectObject(targetDc, hTargetBitmap);

	BitBlt(targetDc,0,0,bmp.bmWidth,bmp.bmHeight,memDc,0,0,SRCCOPY);
	ReversBitmap(pbits,pImage); 

	SelectObject(memDc, hOldBitmap1);
	SelectObject(targetDc, hOldBitmap2);
	DeleteDC(memDc);
	DeleteDC(targetDc);
	DeleteObject(hSourceBitmap);
	delete[] (BYTE*)pbmi;
	return;
}

//-------------------------------------------------------------
//		GetFontData
//-------------------------------------------------------------
void GetFontData(IMAGE_DATA* pImage,WORD id) 
{
	HDC hDC;
	BITMAP  bmp;
	BITMAPINFO *pbmi;
	void *pbits=NULL;

	pbmi = (BITMAPINFO*)new BYTE[sizeof(BITMAPINFO)+sizeof(RGBQUAD)*2];
	ZeroMemory(&pbmi->bmiHeader, sizeof(pbmi->bmiHeader));

	HBITMAP hSourceBitmap = LoadBitmap(g_hInst, MAKEINTRESOURCE(id));
	GetObject( hSourceBitmap, sizeof(BITMAP), &bmp);

	pbmi->bmiHeader.biSize        = sizeof(BITMAPINFOHEADER);
	pbmi->bmiHeader.biWidth       = bmp.bmWidth;
	pbmi->bmiHeader.biHeight      = -bmp.bmHeight;
	pbmi->bmiHeader.biBitCount    = bmp.bmBitsPixel;
	pbmi->bmiHeader.biCompression = BI_RGB;
	pbmi->bmiHeader.biPlanes      = bmp.bmPlanes;
	pbmi->bmiHeader.biSizeImage =
		(((bmp.bmWidth
			* bmp.bmBitsPixel+31)>>3)
				& 0xfffffffc)
					* bmp.bmHeight;

	pImage->dwWidth     = bmp.bmWidth;
	pImage->dwHeight    = bmp.bmHeight;
	pImage->dwBitsPixel = bmp.bmBitsPixel;
	pImage->dwPlanes    = bmp.bmPlanes;
	pImage->dwImageSize = pbmi->bmiHeader.biSizeImage;

	pbmi->bmiColors[0].rgbRed	= 0;
	pbmi->bmiColors[0].rgbGreen = 0;
	pbmi->bmiColors[0].rgbBlue	= 0;
	pbmi->bmiColors[1].rgbRed	= 255;
	pbmi->bmiColors[1].rgbGreen = 255;
	pbmi->bmiColors[1].rgbBlue	= 255;


	hDC = GetDC(NULL);
	HBITMAP hTargetBitmap = CreateDIBSection(hDC, pbmi, DIB_RGB_COLORS, &pbits, NULL, 0);
	ReleaseDC(NULL, hDC);

	HDC memDc    = CreateCompatibleDC(NULL);
	HDC targetDc = CreateCompatibleDC(NULL);

	HBITMAP hOldBitmap1 = (HBITMAP)SelectObject(memDc, hSourceBitmap);
	HBITMAP hOldBitmap2 = (HBITMAP)SelectObject(targetDc, hTargetBitmap);

	BitBlt(targetDc,0,0,bmp.bmWidth,bmp.bmHeight,memDc,0,0,SRCCOPY);
	DeletePading(pbits,pImage);

	SelectObject(memDc, hOldBitmap1);
	SelectObject(targetDc, hOldBitmap2);
	DeleteDC(memDc);
	DeleteDC(targetDc);
	DeleteObject(hSourceBitmap);
	delete[] (BYTE*)pbmi;
	return;
}

//-------------------------------------------------------------
//		DeletePading
//-------------------------------------------------------------
void DeletePading(void *pBits,IMAGE_DATA* image)
{
	BYTE* pSource;
	TCHAR* pDestW;

	int pixel=1;
	int border = ((image->dwWidth *  pixel +31)>>3)& 0xfffffffc;
	int realByte=getRealByte(image->dwWidth);
	int padingByte=	 border-realByte;


	image->pImageByte  = NULL; 
	image->pImageWbyte = new TCHAR[image->dwImageSize];

	int imageSize=image->dwImageSize;
	pSource = (BYTE*)pBits;
	pDestW  = image->pImageWbyte;
	for(int i=0,cnt=1; i<imageSize; pSource++,pDestW++,i++)
	{
		*pDestW=~(*pSource);
		*pDestW&=0x00FF;
		if(cnt==realByte)
		{
			cnt=1;
			i+=padingByte;
			pSource= pSource+padingByte;
			continue;
		}
		cnt++;
	}
	
	image->dwImageSize = realByte * (image->dwImageSize / border);
}


//-------------------------------------------------------------
//		getPadingByte   
//-------------------------------------------------------------
int getPadingByte(int width)
{
	int xbyte,border;
	int pixel=1;	//monochro

	border = ((width *  pixel +31)>>3)& 0xfffffffc;
	xbyte=getRealByte(width) ;
	return border-xbyte;
}
//-------------------------------------------------------------
//		getRealByte 
//-------------------------------------------------------------
int getRealByte(int width)
{
	int xbyte;

	if(width%8)
	{
		xbyte=width/8+1;
	}
	else
	{
		xbyte=width/8;
	}
    return xbyte;
}
//-------------------------------------------------------------
//		ReversBitmap 
//-------------------------------------------------------------
void ReversBitmap(void *pBits,IMAGE_DATA* image)
{
	BYTE* pSource;
	BYTE* pDestB;
	TCHAR* pDestW;

	image->pImageByte  = new BYTE[image->dwImageSize];
	image->pImageWbyte = new TCHAR[image->dwImageSize];

	int imageSize=image->dwImageSize;
	pSource = (BYTE*)pBits;
	pDestB  = image->pImageByte;
	pDestW  = image->pImageWbyte;

	for(int i=0; i<imageSize;i++, pSource++, pDestB++,pDestW++)
	{
		*pDestB=~(*pSource);  
		*pDestW=~(*pSource);  
		*pDestW&=0x00FF; 
	}
}

//-------------------------------------------------------------
//		DeleteImageData
//-------------------------------------------------------------
void DeleteImageData(IMAGE_DATA* image)
{
	if(image->pImageByte!=NULL)
	{
		delete[] image->pImageByte;
		image->pImageByte=NULL;
	}

	if(image->pImageWbyte!=NULL)
	{
		delete[] image->pImageWbyte;
		image->pImageWbyte=NULL;
	}

}

//-------------------------------------------------------------
//		SendCancel
//-------------------------------------------------------------
DWORD SendCancel()
{
	TCHAR CAN[] = {0x18,0x00};

	DWORD dwResult=PRNGetLastError();
	PRNTextOut(wcslen(CAN), CAN);
	return dwResult;
}

//-------------------------------------------------------------
//		GetPrintContinuation
//-------------------------------------------------------------
DWORD GetPrintContinuation(DWORD* pContinuation)
{ 
	DWORD dwPaperType; 
	DWORD dwDepth; 
	DWORD dwSpeed; 
	DWORD dwAutoloading; 
	DWORD dwLoadingValue;
	DWORD dwPreHeat; 
	DWORD dwPrintContinuation;

	DWORD dwResult=PRNGetPrinterProperty (
						&dwPaperType, 
						&dwDepth, 
						&dwSpeed, 
						&dwAutoloading, 
						&dwLoadingValue, 
						&dwPreHeat, 
						&dwPrintContinuation);

	*pContinuation=dwPrintContinuation;
	return 	dwResult;
}

//-------------------------------------------------------------
//		CheckRegistError
//-------------------------------------------------------------
DWORD CheckRegistError(DWORD dwResult)
{ 
	DWORD dwRegistResult=PRN_NORMAL;
	switch(dwResult)
	{
		case PRN_NORMAL:
		case PRN_NOTFOUND:
		case PRN_NOTCHANGE:
		case PRN_FILE_NOTEXIST:
		case PRN_FILEFORMAT_ERROR:
		case PRN_FILEOPEN_ERROR:
		case PRN_HARDWARE_ERROR:
		case PRN_PLATEN_OPEN:
		case PRN_PAPER_END:
		case PRN_VDETP_OCCURRED:
		case PRN_SUSPEND_OCCURRED:
		case PRN_HEADTEMP_ERROR:
		case PRN_AUTOLOADING:
		case PRN_FEEDKEY_ERROR:
			dwRegistResult=PRN_NORMAL;
			break;

		case PRN_NOTOPEN:
		case PRN_DRIVER_NOTEXIST:
		case PRN_ALREADY_OPEN:
		case PRN_PARAMETER_ERROR:
		default:
			dwRegistResult=dwResult;
			break;	
	}
	return dwRegistResult;
}

//-------------------------------------------------------------
//		WaitForPrintOut
//-------------------------------------------------------------
DWORD WaitForPrintOut(HWND hWnd, DWORD dwContinuation)
{
	TCHAR ESC_E[]   = {ESC,'E',0x00};

	DWORD dwResult=PRN_NORMAL;
	do
	{
		dwResult=PRNTextOut(wcslen(ESC_E), ESC_E);
		if( (dwContinuation ==TRUE) && 
			 ( (dwResult==PRN_PAPER_END)	
			|| (dwResult==PRN_VDETP_OCCURRED)
			|| (dwResult==PRN_SUSPEND_OCCURRED)	
			|| (dwResult==PRN_HEADTEMP_ERROR) ) )
		{
			int ret=MessageBox(hWnd, errorMsg[dwResult],c_szCONFERM,
					MB_RETRYCANCEL | MB_ICONINFORMATION | MB_TOPMOST);
			if(ret== IDRETRY)
			{
				continue;
			}
			else
			{
				SendCancel();
				return RETRY_CANCEL;
			}
		}
		else if( dwResult!=PRN_NORMAL) 
		{
			dwResult=SendCancel();
			return dwResult;
		}
	}
	while(dwResult!=PRN_NORMAL);
	return dwResult;
}
//----------------------End Of File------------------------------------------
