/******************************************************************************
*  prnStamp.cpp                                                               *
*                                                                             *
*  Project : Printer Demo Application for DT-9700/IT-3000                     *
*  Date    : 2004-03-05                                                       *
*  Version : 1.0.0                                                            *
*                                                                             *
*  Copyright (C) 2004, CASIO COMPUTER CO.,Ltd. All rights reserved.           *
*                                                                             *
******************************************************************************/
#include    "PrinterLibSampleVC.h"


//-------------------------------------------------------------
//		StampWndProc
//-------------------------------------------------------------
LRESULT CALLBACK WINAPI	StampWndProc(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam)
{
	RECT rc;
	int x, y,nWidth, nHeight; 
	int ret;
	int resourceId=	IDB_STAMP;
	switch (message) 
	{ 
		case WM_INITDIALOG:
			WNDCLASS	wc;
			wc.style			= CS_HREDRAW | CS_VREDRAW;
			wc.lpfnWndProc		= (WNDPROC)GraphicViewProc;
			wc.cbClsExtra		= 0;
			wc.cbWndExtra		= 0;
			wc.hInstance		= g_hInst;
			wc.hIcon			= NULL;
			wc.hCursor			= 0;
			wc.hbrBackground	= (HBRUSH) GetStockObject(GRAY_BRUSH);
			wc.lpszMenuName		= 0;
			wc.lpszClassName	= TEXT("STAMP_VIEW");
			ret=RegisterClass(&wc);

			GetClientRect(hWnd, &rc); 
			x = rc.left+20; 
			y = rc.top+20; 
			nWidth=rc.right-rc.left-40; 
			nHeight=110; 
			CreateWindow(TEXT("STAMP_VIEW"), TEXT("STAMP_VIEW"), 
					WS_CHILD|WS_VISIBLE,
					x,y,nWidth,nHeight, hWnd, (HMENU)NULL, g_hInst,(LPVOID)&resourceId);
			break;

		case WM_COMMAND:
			switch (LOWORD(wParam)) 
			{
				case IDM_STAMP_REGIST:
					RegisterStamp(hWnd);
					break;

				case IDM_STAMP_PRINT:
					PrintStamp(hWnd);
					break;

				case   IDCANCEL:
					EndDialog(hWnd, TRUE); 
					break;
			}
			break;

		case WM_CLOSE:
			EndDialog(hWnd, TRUE); 
			break;

		case WM_DESTROY:
			EndDialog(hWnd, TRUE); 
			return 0;
		default:
			return FALSE;
	}
	
	return TRUE;
}

//-------------------------------------------------------------
//		RegisterStamp
//-------------------------------------------------------------
void RegisterStamp(HWND hWnd)
{
	DWORD dwResult;

	dwResult=RegisterStampExec( );
	if(dwResult !=PRN_NORMAL)
	{
		MessageBox(hWnd, errorMsg[dwResult],c_szCONFERM,
			MB_OK| MB_ICONERROR | MB_TOPMOST);
		return;
	}

	MessageBox(hWnd, c_szREGISTFINISH,c_szCONFERM,MB_OK);
	return;
}


//-------------------------------------------------------------
//		RegisterStampExec
//-------------------------------------------------------------
DWORD RegisterStampExec()
{
	DWORD dwResult = PRN_NORMAL;
	TCHAR  ESC_K[] = {ESC,'K',1,1,0};

	dwResult=PRNOpen();
	if(dwResult!=PRN_NORMAL)
	{
		return dwResult;
	}

	dwResult = PRNGetStatus();
	if(dwResult!=PRN_NORMAL)
	{
		SendCancel();
	}

	struct IMAGE_DATA image;
	GetImageData(&image,IDB_STAMP); 

	TCHAR *pBuff = new TCHAR[image.dwImageSize+wcslen(ESC_K)];
	memcpy(&pBuff[wcslen(ESC_K)], image.pImageWbyte, image.dwImageSize*2);

	pBuff[0] = ESC;
	pBuff[1] = 'K';
	pBuff[2] = (USHORT)(image.dwWidth /8);
	if(image.dwWidth % 8)
	{
		pBuff[2] += 1; 	
	}
	pBuff[3] = (USHORT)(image.dwHeight);

	dwResult = PRNTextOut(wcslen(ESC_K)+image.dwImageSize, pBuff);

	dwResult=CheckRegistError(dwResult);
	PRNClose();
	delete[] pBuff;
	DeleteImageData(&image);
	return dwResult;
}

//-------------------------------------------------------------
//		PrintStamp
//-------------------------------------------------------------
void PrintStamp(HWND hWnd)
{
	DWORD dwResult;

	dwResult=PrintStampExec(hWnd);
	if(dwResult==RETRY_CANCEL)
	{
		return;
	}
	if(dwResult !=PRN_NORMAL)
	{
		MessageBox(hWnd, errorMsg[dwResult],c_szCONFERM,
			MB_OK | MB_ICONERROR| MB_TOPMOST);
		return;
	}

	MessageBox(hWnd, c_szPRINTFINISH,c_szCONFERM,MB_OK);
	return;
}


//-------------------------------------------------------------
//		PrintStampExec
//-------------------------------------------------------------
DWORD PrintStampExec(HWND hWnd)
{
	DWORD dwResult=PRN_NORMAL;
	DWORD dwWidth;
	DWORD dwContinuation;

	TCHAR ESC_G[] = {ESC,'G',0x0D,0x00};

	dwResult=PRNOpen();
	if(dwResult!=PRN_NORMAL)
	{
		return dwResult; 
	}

	dwResult=PRNGetPaperWidth(&dwWidth);
	if(dwResult!=PRN_NORMAL)
	{
		PRNClose();
		return dwResult;
	}
	
	if(dwWidth==PAPER_WIDTH58)
	{
		PRNClose();
		return AP_ERROR_WIDTH;
	}

	dwResult=GetPrintContinuation(&dwContinuation);
	if(dwResult!=PRN_NORMAL)
	{
		PRNClose();
		return dwResult;
	}

	dwResult = PRNTextOut(wcslen(ESC_G),ESC_G);
	if((dwResult!=PRN_NORMAL) && (dwContinuation!=TRUE))
	{
		dwResult=SendCancel();
		PRNClose();
		return dwResult;
	}

	dwResult= WaitForPrintOut(hWnd,dwContinuation);
	PRNClose();
	return dwResult;
}

//-------------------------- END OF FILE -------------------------------------------------------------