using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Calib;

namespace PrinterLibSampleWMCS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>

        string strInputCharANK = ((char)27).ToString() + "Y" + ((char)1).ToString();
        // This is ESC code : Input Character code select
        // Format is ESC Y n
        //						n : 00h(Unicode), 01h(ANK 0020h - 00FFh)
        string strBuff = ((char)32).ToString();
        string strCR = ((char)13).ToString();	// Set Carriage Return code

        private void button1_Click(object sender, EventArgs e)
        {
            int nRet;
            int i;
            int dwPaperType = new int();
            int dwDepth = new int();
            int dwSpeed = new int();
            int dwAutoloading = new int();
            int dwLoadingValue = new int();
            int dwPreHeat = new int();
            int dwPrintContinuation = new int();

            //printer driver open
            nRet = PrinterLibNet.Api.PRNOpen();
            if (nRet != PrinterLibNet.Def.PRN_NORMAL)
            {
                ShowErrorMessage("PRNOpen", nRet);
                return;
            }
            //check printer state
            nRet = PrinterLibNet.Api.PRNGetPrinterProperty(ref dwPaperType, ref dwDepth, ref dwSpeed, ref dwAutoloading, ref dwLoadingValue, ref dwPreHeat, ref dwPrintContinuation);
            if (nRet != PrinterLibNet.Def.PRN_NORMAL)
            {
                ShowErrorMessage("PRNGetPrinterProperty", nRet);
                return;
            }
            //print out character code
            nRet = PrinterLibNet.Api.PRNTextOut(0, "PrinterLibNet Sample-PRNTextOut\n\n");
            ShowErrorMessage("PRNTextOut", nRet);

            PrinterLibNet.Api.PRNTextOut(3, strInputCharANK);	// select ANK input character code
            for (i = 32; i < 256; i++)	// print out from 20h to FFh ANK code
            {
                strBuff = ((char)i).ToString();
                nRet = PrinterLibNet.Api.PRNTextOut(1, strBuff);
                ShowErrorMessage("PRNTextOut", nRet);
            }
            //print out carriage return
            nRet = PrinterLibNet.Api.PRNTextOut(1, strCR);

            //close printer driver
            nRet = PrinterLibNet.Api.PRNClose();
            ShowErrorMessage("PRNClose", nRet);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            int nRet;
            int i;
            byte[] ByteWorkBuf1 = new byte[72 * 10];
            int dwPaperType = new int();
            int dwDepth = new int();
            int dwSpeed = new int();
            int dwAutoloading = new int();
            int dwLoadingValue = new int();
            int dwPreHeat = new int();
            int dwPrintContinuation = new int();

            //printer driver open
            nRet = PrinterLibNet.Api.PRNOpen();
            if (nRet != PrinterLibNet.Def.PRN_NORMAL)
            {
                ShowErrorMessage("PRNOpen", nRet);
                return;
            }
            //check printer state
            nRet = PrinterLibNet.Api.PRNGetPrinterProperty(ref dwPaperType, ref dwDepth, ref dwSpeed, ref dwAutoloading, ref dwLoadingValue, ref dwPreHeat, ref dwPrintContinuation);
            if (nRet != PrinterLibNet.Def.PRN_NORMAL)
            {
                ShowErrorMessage("PRNGetPrinterProperty", nRet);
                return;
            }

            for (i = 0; i < ByteWorkBuf1.Length; i++)
            {
                ByteWorkBuf1[i] = 0xff;
            }
            //print out a black quadrangle
            nRet = PrinterLibNet.Api.PRNImageOut(72, 10, 0, ByteWorkBuf1);
            ShowErrorMessage("PRNImageOut", nRet);

            //print out carriage return
            PrinterLibNet.Api.PRNTextOut(1, strCR);

            //print out a bitmap image
            nRet = PrinterLibNet.Api.PRNBMPOut("\\Windows\\BITMAP.bmp");
            ShowErrorMessage("PRNBMPOut", nRet);

            PrinterLibNet.Api.PRNTextOut(1, strCR);
            //close printer driver
            nRet = PrinterLibNet.Api.PRNClose();
            ShowErrorMessage("PRNClose", nRet);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            int nRet;
            int dwPaperType = new int();
            int dwDepth = new int();
            int dwSpeed = new int();
            int dwAutoloading = new int();
            int dwLoadingValue = new int();
            int dwPreHeat = new int();
            int dwPrintContinuation = new int();

            //open printer driver
            nRet = PrinterLibNet.Api.PRNOpen();
            if (nRet != PrinterLibNet.Def.PRN_NORMAL)
            {
                ShowErrorMessage("PRNOpen", nRet);
                return;
            }
            //check printer state
            nRet = PrinterLibNet.Api.PRNGetPrinterProperty(ref dwPaperType, ref dwDepth, ref dwSpeed, ref dwAutoloading, ref dwLoadingValue, ref dwPreHeat, ref dwPrintContinuation);
            if (nRet != PrinterLibNet.Def.PRN_NORMAL)
            {
                ShowErrorMessage("PRNGetPrinterProperty", nRet);
                return;
            }

            //print out display screen
            nRet = PrinterLibNet.Api.PRNPrintScreen();
            ShowErrorMessage("PRNPrintScreen", nRet);

            PrinterLibNet.Api.PRNTextOut(1, strCR);
            //close printer driver
            nRet = PrinterLibNet.Api.PRNClose();
            ShowErrorMessage("PRNClose", nRet);
        }

        private void ShowErrorMessage(string pszCaption, int nStatus)
        {
            string szTitle = "PrinterLib Sample for C#";
            string szMsg;
            //error message
            switch (nStatus)
            {
                case PrinterLibNet.Def.PRN_NORMAL:
                    return;

                case PrinterLibNet.Def.PRN_NOTOPEN:
                    szMsg = "PRN_NOTOPEN";
                    break;

                case PrinterLibNet.Def.PRN_DRIVER_NOTEXIST:
                    szMsg = "PRN_DRIVER_NOTEXIST";
                    break;

                case PrinterLibNet.Def.PRN_ALREADY_OPEN:
                    szMsg = "PRN_ALREADY_OPEN";
                    break;

                case PrinterLibNet.Def.PRN_NOTFOUND:
                    szMsg = "Def.PRN_NOTFOUND";
                    break;

                case PrinterLibNet.Def.PRN_NOTCHANGE:
                    szMsg = "PRN_NOTCHANGE";
                    break;

                case PrinterLibNet.Def.PRN_FILE_NOTEXIST:
                    szMsg = "PRN_FILE_NOTEXIST";
                    break;

                case PrinterLibNet.Def.PRN_FILEFORMAT_ERROR:
                    szMsg = "PRN_FILEFORMAT_ERROR";
                    break;

                case PrinterLibNet.Def.PRN_FILEOPEN_ERROR:
                    szMsg = "PRN_FILEOPEN_ERROR";
                    break;

                case PrinterLibNet.Def.PRN_PARAMETER_ERROR:
                    szMsg = "PRN_PARAMETER_ERROR";
                    break;

                case PrinterLibNet.Def.PRN_HARDWARE_ERROR:
                    szMsg = "PRN_HARDWARE_ERROR";
                    break;

                case PrinterLibNet.Def.PRN_PLATEN_OPEN:
                    szMsg = "PRN_PLATEN_OPEN";
                    break;

                case PrinterLibNet.Def.PRN_PAPER_END:
                    szMsg = "RN_PAPER_END";
                    break;

                case PrinterLibNet.Def.PRN_AUTOLOADING:
                    szMsg = "PRN_AUTOLOADING";
                    break;

                case PrinterLibNet.Def.FUNCTION_UNSUPPORT:
                    szMsg = "FUNCTION_UNSUPPORT";
                    break;

                default:
                    szMsg = "Unknown Status";
                    break;
            }

            // display messagebox
            MessageBox.Show(szMsg, szTitle);
        }
    }
}