/***************************************************************************
 * <Description>
 * SAMMain.cs (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * SAM Card accessing program
 * -------------------------------------------------------------------------
 * <Language>
 * C#.NET
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * IT-9000 WM
 * -------------------------------------------------------------------------
 * Copyright(C)2012 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2012.05.24      CASIO       000000      Original      
 * 
 ***************************************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.VisualBasic;

using Calib;

namespace SAMLibSampleWMCS
{
    public partial class SAMMain : Form
    {
        int i;
        int iRet;
        int dwNumberOfSlot;
        string strCommand;
        int iActBaudRate;
        int dwActualNfc;
        
        // For SAMExchangeData
        // The following send data is dummy.
        int	dwSlotNumber = 0;
        byte [] pbySendSam = new byte[506];
        int	dwSendSam = 10;
        byte [] pbyReceiveSam = new byte[506];

        // For NFCExchangeData
        // The following send data is dummy.
        byte [] pbySendNfc=new byte[261];
        int	dwSendNfc = 10;
        byte [] pbyReceiveNfc=new byte[261];

        // For SAMGetCardResponse
        int[] pdwActualSam = new int[3 + 1]; //3:MAX_NUM_SLOT
        byte[][] pbyTargetSam = new byte[3 + 1][]; //3:MAX_NUM_SLOT

        public SAMMain()
        {
            InitializeComponent();
            for (i = 0; i < pbyTargetSam.Length; i++)
            {
                pbyTargetSam[i] = new byte[506];
            }
            for(i =0;i<10; i++)
            {
                pbySendSam[i] = (byte)(i+1);
                pbySendNfc[i] = (byte)(i+1);
            }
        }
        
        private void SAM_Access()
        {
            int dwActualSam;

            // Modify your SAMExchangeData code.
            textBox1.Text = "SAMExchangeData...start.";
            dwActualSam = 0;
            iRet = SAMLibNet.Api.SAMExchangeData(dwSlotNumber, pbySendSam, dwSendSam, pbyReceiveSam, ref dwActualSam, 0);
            if (iRet != SAMLibNet.Def.SAM_OK)
            {
                strCommand = "SAMExchangeData";
                DispSamErrorMessage(iRet, strCommand);
                return;
            }
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMExchangeData...OK.";

            SystemLibNet.Api.SysSetBuzzerVolume(SystemLibNet.Def.B_SCANEND, SystemLibNet.Def.BUZZERVOLUME_MAX);
            SystemLibNet.Api.SysPlayBuzzer(SystemLibNet.Def.B_SCANEND, SystemLibNet.Def.B_USERDEF, SystemLibNet.Def.B_USERDEF);
        }

        private void NFC_Scan()
        {
            int dwActualSam;

            // NFC POLLING
            textBox1.Text = "NFCPollingCard...start.";
            iRet = NFCLibNet.Api.NFCPollingCard(NFCLibNet.Def.NFC_BAUD_TYPEA |
                                                NFCLibNet.Def.NFC_BAUD_TYPEB |
                                                NFCLibNet.Def.NFC_BAUD_FELICA1 |
                                                NFCLibNet.Def.NFC_BAUD_FELICA2
#if HFTAG_UNSUPPORTED
#else
                                                | NFCLibNet.Def.NFC_BAUD_HFTAGE
#endif
                                                , 10000, IntPtr.Zero, ref iActBaudRate, 0);
            if (iRet != NFCLibNet.Def.NFC_OK)
            {
                strCommand = "NFCPollingCard";
                DispNfcErrorMessage(iRet, strCommand);
                return;
            }
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "NFCPollingCard...OK.";

            // Modify your SAMExchangeData code.
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMExchangeData...start.";
            dwActualSam = 0;
            iRet = SAMLibNet.Api.SAMExchangeData(dwSlotNumber, pbySendSam, dwSendSam, pbyReceiveSam, ref dwActualSam, 0);
            if (iRet != SAMLibNet.Def.SAM_OK)
            {
                strCommand = "SAMExchangeData";
                DispSamErrorMessage(iRet, strCommand);
                return;
            }
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMExchangeData...OK.";

            // Modify your NFCExchangeData code.
            // In case of sample, in order to transmit dummy data, function returns error.
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "NFCExchangeData...start.";
            dwActualNfc = 0;
            iRet = NFCLibNet.Api.NFCExchangeData(pbySendNfc, dwSendNfc, pbyReceiveNfc, ref dwActualNfc, 0);
            if (iRet != NFCLibNet.Def.NFC_OK)
            {
                strCommand = "NFCExchangeData";
                DispNfcErrorMessage(iRet, strCommand);
                return;
            }
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "NFCExchangeData...OK.";


            // Append your SAMExchangeData code.


            // Append your NFCExchangeData code.


            // NFC RADIO OFF
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "NFCRadioOff...start.";
            iRet = NFCLibNet.Api.NFCRadioOff();
            if (iRet != NFCLibNet.Def.NFC_OK)
            {
                strCommand = "NFCRadioOff";
                DispNfcErrorMessage(iRet, strCommand);
                return;
            }
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "NFCRadioOff...OK.";

            button1.Enabled = true;
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "NFC scanning...OK.";
            SystemLibNet.Api.SysSetBuzzerVolume(SystemLibNet.Def.B_SCANEND, SystemLibNet.Def.BUZZERVOLUME_MAX);
            SystemLibNet.Api.SysPlayBuzzer(SystemLibNet.Def.B_SCANEND, SystemLibNet.Def.B_USERDEF, SystemLibNet.Def.B_USERDEF);
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            button1.Enabled = false;

            if (comboBox1.SelectedIndex == 0)
            {
                SAM_Access();
            }
            else
            {
                NFC_Scan();
            }

            button1.Enabled = true;
        }

        private void Button2_Click(object sender, EventArgs e)
        {
            Common.ShowTaskbar();
            this.Close();
        }

        private void SAMMain_Load(object sender, EventArgs e)
        {
            Common.HideTaskbar();
            comboBox1.SelectedIndex = 0;

            // NFC OPEN
            textBox1.Text = "NFCOpen...start.";
            iRet = NFCLibNet.Api.NFCOpen(IntPtr.Zero);
            if (iRet != NFCLibNet.Def.NFC_OK)
            {
                strCommand = "NFCOpen";
                DispNfcErrorMessage(iRet, strCommand);
                return;
            }
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "NFCOpen...OK.";

            // SAM OPEN
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMOpen...start." ;
            iRet = SAMLibNet.Api.SAMOpen();
            if (iRet != SAMLibNet.Def.SAM_OK)
            {
                strCommand = "SAMOpen";
                DispSamErrorMessage(iRet, strCommand);
                return;
            }
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMOpen...OK." ;

            // SAM GET NUMBER OF SLOT
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMGetNumberOfSlot...start." ;
            iRet = SAMLibNet.Api.SAMGetNumberOfSlot(ref dwNumberOfSlot);
            if (iRet != SAMLibNet.Def.SAM_OK)
            {
                strCommand = "SAMGetNumberOfSlot";
                DispSamErrorMessage(iRet, strCommand);
                return;
            }
            textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMGetNumberOfSlot...OK." ;
            for (i = 1; i <= dwNumberOfSlot; i++)
            {
                comboBox2.Items.Add(i);

                // SAM POWERUP
                textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMPowerUpCard(" + i + ")...start." ;
                iRet = SAMLibNet.Api.SAMPowerUpCard(i, SAMLibNet.Def.SAM_POWER_ISO);
                if (iRet == SAMLibNet.Def.SAM_OK)
                {
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMPowerUpCard(" + i + ")...OK." ;

                    // SAM GET CARD RESPONSE
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMGetCardResponse(" + i + ")...start.";
                    pdwActualSam[i] = 0;
                    iRet = SAMLibNet.Api.SAMGetCardResponse(i, pbyTargetSam[i], ref pdwActualSam[i], 0);
                    if (iRet == SAMLibNet.Def.SAM_OK)
                    {
                        textBox1.Text = textBox1.Text + ControlChars.NewLine + "SAMGetCardResponse(" + i + ")...OK." ;
                    }
                    else
                    {
                        strCommand = "SAMGetCardResponse(" + i + ")";
                        DispSamErrorMessage(iRet, strCommand);
                    }
                }
                else
                {
                    strCommand = "SAMPowerUpCard(" + i + ")";
                    DispSamErrorMessage(iRet, strCommand);
                }
            }

            comboBox2.SelectedIndex = 0;
            dwSlotNumber = 1;

            //DispHex
            label4.Text = "";
            for (i = 0; i < pdwActualSam[dwSlotNumber]; i++)
            {
                label4.Text = label4.Text + pbyTargetSam[dwSlotNumber][i].ToString("X2") + " ";
            }
        }
        
        private void DispNfcErrorMessage(int iRet, string strCommand)
        {
            switch (iRet)
            {
                case NFCLibNet.Def.NFC_PON:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : NFC already opend.";
                    break;
                case NFCLibNet.Def.NFC_NOT_DEVICE:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : NFC driver error.";
                    break;
                case NFCLibNet.Def.NFC_ERROR_INVALID_ACCESS:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Device exclusive error.";
                    break;
                case NFCLibNet.Def.NFC_ERROR_MODULE:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : module not responce error.";
                    break;
                case NFCLibNet.Def.NFC_POF:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : NFC not open error.";
                    break;
                case NFCLibNet.Def.NFC_PRM:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : NFC parameter error.";
                    break;
                case NFCLibNet.Def.NFC_ERROR_TIMEOUT:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Timeout error.";
                    break;
                case NFCLibNet.Def.NFC_ERROR_CALLBACK:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Call back function error.";
                    break;
                case NFCLibNet.Def.NFC_ERROR_STOP:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Stop error by stop function.";
                    break;
                case NFCLibNet.Def.NFC_NOT_ACTIVATION:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Card don't start error.";
                    break;
                default:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Other error.";
                    break;
            }
            SystemLibNet.Api.SysSetBuzzerVolume(SystemLibNet.Def.B_WARNING, SystemLibNet.Def.BUZZERVOLUME_MAX);
            SystemLibNet.Api.SysPlayBuzzer(SystemLibNet.Def.B_WARNING, SystemLibNet.Def.B_USERDEF, SystemLibNet.Def.B_USERDEF);

            //NFCLibNet.Api.NFCClose();
            button1.Enabled = true;
        }

        private void DispSamErrorMessage(int iRet, string strCommand)
        {
            switch (iRet)
            {
                case SAMLibNet.Def.SAM_PON:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : SAM already opened.";
                    break;
                case SAMLibNet.Def.SAM_NOT_DEVICE:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : SAM driver error.";
                    break;
                case SAMLibNet.Def.SAM_ERROR_INVALID_ACCESS:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Device exclusive error.";
                    break;
                case SAMLibNet.Def.SAM_ERROR_MODULE:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : SAM module not responce error.";
                    break;
                case SAMLibNet.Def.SAM_POF:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : SAM not open error.";
                    break;
                case SAMLibNet.Def.SAM_PRM:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : SAM parameter error.";
                    break;
                case SAMLibNet.Def.SAM_ERROR_TIMEOUT:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Timeout error.";
                    break;
                case SAMLibNet.Def.SAM_NOT_ACTIVATION:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Card don't start error.";
                    break;
                case SAMLibNet.Def.SAM_ERROR_SUSPEND:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Suspend error.";
                    break;
                case SAMLibNet.Def.SAM_ERROR_RESPONSE:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Card response error.";
                    break;
                case SAMLibNet.Def.SAM_ERROR_NOCARD:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Card does not exist.";
                    break;
                default:
                    textBox1.Text = textBox1.Text + ControlChars.NewLine + strCommand + " : Other error.";
                    break;
            }
            SystemLibNet.Api.SysSetBuzzerVolume(SystemLibNet.Def.B_WARNING, SystemLibNet.Def.BUZZERVOLUME_MAX);
            SystemLibNet.Api.SysPlayBuzzer(SystemLibNet.Def.B_WARNING, SystemLibNet.Def.B_USERDEF, SystemLibNet.Def.B_USERDEF);

            //NFCLibNet.Api.NFCClose();
            button1.Enabled = true;
        }

        private void SAMMain_Closed(object sender, EventArgs e)
        {
            NFCLibNet.Api.NFCClose();
            SAMLibNet.Api.SAMClose();
        }

        private void comboBox2_change(object sender, EventArgs e)
        {
            //DispHex
            label4.Text = "";
            dwSlotNumber = comboBox2.SelectedIndex + 1;

            for (i = 0; i < pdwActualSam[dwSlotNumber]; i++)
            {
                label4.Text = label4.Text + pbyTargetSam[dwSlotNumber][i].ToString("X2") + " ";
            }
        }
    }
}