/***************************************************************************
 * <Description>
 * SAMMain.cpp (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * SAM Card accessing program
 * -------------------------------------------------------------------------
 * <Language>
 * Visual C++
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * IT-9000 WM
 * -------------------------------------------------------------------------
 * Copyright(C)2012 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2012.05.24      CASIO       000000      Original      
 * 
 ***************************************************************************/
//	000000__	Original
//	__000000	Original

#include <windows.h>
#include "resource.h"
#include <aygshell.h>

//	000000__	Original
//  CASIO Libaray
#include "SAMLib.h"
#include "NFCLib.h"
#include "SystemLib.h"
//	__000000	Original

#define MAX_NUM_SLOT 3

BOOL DlgNFC(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL SAM_Access(HWND hDlg);
BOOL NFC_Scan(HWND hDlg);
BOOL OutputString(HWND hWnd, TCHAR *szString);
BOOL OutputNfcErrorMessage(HWND hWnd, int iRet, TCHAR *szFunction);
BOOL OutputSamErrorMessage(HWND hWnd, int iRet, TCHAR *szFunction);
BOOL DispHex(HWND hWnd, BYTE *pbyData, DWORD dwLen);

int		iRet;
DWORD	dwNumberOfSlot = 0;

// For display status information
TCHAR	szMessage[500];
TCHAR	szMessage2[500];

// For NFCExchangeData
// The following send data is dummy.
BYTE	pbySendNfc[261] = {0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A};
DWORD	dwSendNfc = 10;
BYTE	pbyReceiveNfc[261] = {0};

// For SAMExchangeData
// The following send data is dummy.
DWORD	dwSlotNumber = 0;
BYTE	pbySendSam[506] = {0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A};
DWORD	dwSendSam = 10;
BYTE	pbyReceiveSam[506] = {0};

// For SAMGetCardResponse
DWORD	pdwActualSam[MAX_NUM_SLOT+1] = {0};
BYTE	pbyTargetSam[MAX_NUM_SLOT+1][506] = {0};

int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
			LPWSTR lpCmdLine, int nShowCmd)
{
	DialogBoxW(hInstance, MAKEINTRESOURCE( IDD_SAM ), NULL, DlgNFC);
	return 0;
}

BOOL DlgNFC(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	int i = 0;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		{
			RECT rc;
			#define MENU_HEIGHT 26

            // Create a Done button and size it.  
            SHINITDLGINFO shidi;
            shidi.dwMask = SHIDIM_FLAGS;
			shidi.dwFlags = SHIDIF_SIPDOWN | SHIDIF_FULLSCREENNOMENUBAR ;
            shidi.hDlg = hDlg;
            SHInitDialog(&shidi);
				
			GetWindowRect(hDlg, &rc);
			MoveWindow(hDlg, rc.left, rc.top, rc.right, rc.bottom+MENU_HEIGHT, TRUE);

			SetForegroundWindow(hDlg);
			SHFullScreen(hDlg, SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON | SHFS_HIDESTARTICON);	
		}

		SendMessage(GetDlgItem(hDlg,IDC_MODE), CB_ADDSTRING, 0, (LPARAM)_T("SAM"));
		SendMessage(GetDlgItem(hDlg,IDC_MODE), CB_ADDSTRING, 0, (LPARAM)_T("NFC & SAM"));
		SendMessage(GetDlgItem(hDlg,IDC_MODE), CB_SETCURSEL, 0, 0);

		// NFC OPEN
		OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("NFCOpen...start.\r\n"));

		iRet = NFCOpen(NULL);

		if (iRet != NFC_OK)
		{
			OutputNfcErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("NFCOpen"));
			return (INT_PTR)TRUE;
		}
		OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("NFCOpen...OK.\r\n"));

		// SAM OPEN
		OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("SAMOpen...start.\r\n"));

		iRet = SAMOpen();

		if (iRet != SAM_OK)
		{
			OutputSamErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("SAMOpen"));
			return (INT_PTR)TRUE;
		}
		OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("SAMOpen...OK.\r\n"));

		// SAM GET NUMBER OF SLOT
		OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("SAMGetNumberOfSlot...start.\r\n"));

		iRet = SAMGetNumberOfSlot(&dwNumberOfSlot);

		if (iRet != SAM_OK)
		{
			OutputSamErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("SAMGetNumberOfSlot"));
			return (INT_PTR)TRUE;
		}
		OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("SAMGetNumberOfSlot...OK.\r\n"));

		for (i=1 ; i<=(int)dwNumberOfSlot ; i++)
		{
			ZeroMemory(szMessage, sizeof(szMessage));
			wsprintf(szMessage, _T("%d"), i);
			SendMessage(GetDlgItem(hDlg,IDC_SLOT), CB_ADDSTRING, 0, (LPARAM)szMessage);

			// SAM POWERUP
			ZeroMemory(szMessage, sizeof(szMessage));
			wsprintf(szMessage, _T("SAMPowerUpCard(%d)...start.\r\n"), i);
			OutputString(GetDlgItem(hDlg,IDC_STATUS), szMessage);

			iRet = SAMPowerUpCard(i, SAM_POWER_ISO);

			if (iRet == SAM_OK)
			{
				ZeroMemory(szMessage, sizeof(szMessage));
				wsprintf(szMessage, _T("SAMPowerUpCard(%d)...OK.\r\n"), i);
				OutputString(GetDlgItem(hDlg,IDC_STATUS), szMessage);

				ZeroMemory(szMessage, sizeof(szMessage));
				wsprintf(szMessage, _T("SAMGetCardResponse(%d)...start.\r\n"), i);
				OutputString(GetDlgItem(hDlg,IDC_STATUS), szMessage);

				// SAM GET CARD RESPONSE
				pdwActualSam[i] = 0;
				iRet = SAMGetCardResponse(i, pbyTargetSam[i], &pdwActualSam[i], 0);

				if (iRet == SAM_OK)
				{
					ZeroMemory(szMessage, sizeof(szMessage));
					wsprintf(szMessage, _T("SAMGetCardResponse(%d)...start.\r\n"), i);
					OutputString(GetDlgItem(hDlg,IDC_STATUS), szMessage);
				}
				else
				{
					OutputSamErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("SAMGetCardResponse"));
				}
			}
			else
			{
				OutputSamErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("SAMPowerUpCard"));
			}
		}
		SendMessage(GetDlgItem(hDlg,IDC_SLOT), CB_SETCURSEL, 0, 0);
		dwSlotNumber = 1;
		DispHex(GetDlgItem(hDlg,IDC_ATR), pbyTargetSam[dwSlotNumber], pdwActualSam[dwSlotNumber]);
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_EXIT:
			EndDialog(hDlg, LOWORD(wParam));
			break;

		case IDC_START:
			if (SendMessage(GetDlgItem(hDlg, IDC_MODE), CB_GETCURSEL,0 ,0 ) == 0)
			{
				SAM_Access(hDlg);
			}
			else
			{
				NFC_Scan(hDlg);
			}
			break;
		case IDC_SLOT:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				dwSlotNumber = SendMessage(GetDlgItem(hDlg, IDC_SLOT), CB_GETCURSEL,0 ,0) + 1;

				DispHex(GetDlgItem(hDlg,IDC_ATR), pbyTargetSam[dwSlotNumber], pdwActualSam[dwSlotNumber]);
			}
			break;
		}
		break;
	case WM_CLOSE:
		EndDialog(hDlg, LOWORD(wParam));
		break;
	case WM_DESTROY:
		// SAM POWERDOWN
		for (i=1 ; i<=(int)dwNumberOfSlot ; i++)
		{
			SAMPowerDownCard(i, 0);
		}
		// SAM CLOSE
		SAMClose();
		// NFC CLOSE
		NFCClose();
		break;
	}
	return FALSE;
}

BOOL SAM_Access(HWND hDlg)
{
	DWORD dwActualSam = 0;
	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),false);
	SetDlgItemText(hDlg, IDC_STATUS,  _T(""));
	UpdateWindow(hDlg);

	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("SAMExchangeData...start.\r\n"));


	// Modify your SAMExchangeData code.
	dwActualSam = 0;
	iRet = SAMExchangeData(dwSlotNumber, pbySendSam, dwSendSam, pbyReceiveSam, &dwActualSam, 0);


	if (iRet != SAM_OK)
	{
		OutputSamErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("SAMExchangeData"));
		EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
		return FALSE;
	}
	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("SAMExchangeData...OK.\r\n"));

	SysSetBuzzerVolume(B_SCANEND, BUZZERVOLUME_MAX);
	SysPlayBuzzer(B_SCANEND, B_USERDEF, B_USERDEF);

	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
	return TRUE;
}

BOOL NFC_Scan(HWND hDlg)
{
	DWORD dwActBaudRate = 0;
	DWORD dwActualSam = 0;
	DWORD dwActualNfc = 0;

	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),false);
	SetDlgItemText(hDlg, IDC_STATUS,  _T(""));
	UpdateWindow(hDlg);

	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("NFCPollingCard...start.\r\n"));

	// NFC POLLING
	iRet = NFCPollingCard(NFC_BAUD_TYPEA | 
						  NFC_BAUD_TYPEB | 
						  NFC_BAUD_FELICA1 | 
						  NFC_BAUD_FELICA2 | 
						  NFC_BAUD_HFTAG,
						  10000 , 
						  NULL ,
						  &dwActBaudRate,0);

	if (iRet != NFC_OK)
	{
		OutputNfcErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("NFCPollingCard"));
		EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
		return FALSE;
	}

	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("NFCPollingCard...OK.\r\n"));

	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("SAMExchangeData...start.\r\n"));


	// Modify your SAMExchangeData code.
	dwActualSam = 0;
	iRet = SAMExchangeData(dwSlotNumber, pbySendSam, dwSendSam, pbyReceiveSam, &dwActualSam, 0);


	if (iRet != SAM_OK)
	{
		OutputSamErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("SAMExchangeData"));
		EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
		return FALSE;
	}
	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("SAMExchangeData...OK.\r\n"));

	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("NFCExchangeData...start.\r\n"));

	// Modify your NFCExchangeData code.
	// In case of sample, in order to transmit dummy data, function returns error.
	dwActualNfc = 0;
	iRet = NFCExchangeData(pbySendNfc, dwSendNfc, pbyReceiveNfc, &dwActualNfc, 0);


	if (iRet != NFC_OK)
	{
		OutputNfcErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("NFCExchangeData"));
		EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
		return FALSE;
	}
	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("NFCExchangeData...OK.\r\n"));


	// Append your SAMExchangeData code.


	// Append your NFCExchangeData code.


	// NFC RADIO OFF
	iRet = NFCRadioOff();

	if (iRet != NFC_OK)
	{
		OutputNfcErrorMessage(GetDlgItem(hDlg,IDC_STATUS), iRet, _T("NFCRadioOff"));
		EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
		return FALSE;
	}

	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("NFCRadioOff...OK.\r\n"));


	SysSetBuzzerVolume(B_SCANEND, BUZZERVOLUME_MAX);
	SysPlayBuzzer(B_SCANEND, B_USERDEF, B_USERDEF);

	OutputString(GetDlgItem(hDlg,IDC_STATUS), _T("NFC scanning...OK.\r\n"));
	EnableWindow(GetDlgItem(hDlg,IDC_NFC_SCAN),true);
	return TRUE;
}


BOOL OutputNfcErrorMessage(HWND hWnd, int iRet, TCHAR *szFunction)
{
	int nLen = sizeof(szMessage)/sizeof(TCHAR);

	ZeroMemory(szMessage, sizeof(szMessage));
	ZeroMemory(szMessage2, sizeof(szMessage2));

	switch (iRet)
	{
	case NFC_PON:
		StringCchCopy(szMessage, nLen, _T("NFC already opened.\r\n"));
		break;
	case NFC_NOT_DEVICE:
		StringCchCopy(szMessage, nLen, _T("NFC driver error.\r\n"));
		break;
	case NFC_ERROR_INVALID_ACCESS:
		StringCchCopy(szMessage, nLen, _T("Device exclusive error.\r\n"));
		break;
	case NFC_ERROR_MODULE:
		StringCchCopy(szMessage, nLen, _T("NFC module not responce error.\r\n"));
		break;
	case NFC_POF:
		StringCchCopy(szMessage, nLen, _T("NFC not open error.\r\n"));
		break;
	case NFC_PRM:
		StringCchCopy(szMessage, nLen, _T("NFC parameter error.\r\n"));
		break;
	case NFC_ERROR_TIMEOUT:
		StringCchCopy(szMessage, nLen, _T("Timeout error.\r\n"));
		break;
	case NFC_ERROR_CALLBACK:
		StringCchCopy(szMessage, nLen, _T("Call back function error.\r\n"));
		break;
	case NFC_ERROR_STOP:
		StringCchCopy(szMessage, nLen, _T("Stop error by stop function.\r\n"));
		break;
	case NFC_NOT_ACTIVATION:
		StringCchCopy(szMessage, nLen, _T("Card don't start error.\r\n"));
		break;
	default:
		StringCchCopy(szMessage, nLen, _T("Other error.\r\n"));
		break;
	}
	wsprintf(szMessage2, _T("%s : %s\r\n"), szFunction, szMessage);
	OutputString(hWnd, szMessage2);

	SysSetBuzzerVolume(B_WARNING, BUZZERVOLUME_MAX);
	SysPlayBuzzer(B_WARNING, B_USERDEF, B_USERDEF);

	return TRUE;
}


BOOL OutputSamErrorMessage(HWND hWnd, int iRet, TCHAR *szFunction)
{
	int nLen = sizeof(szMessage)/sizeof(TCHAR);

	ZeroMemory(szMessage, sizeof(szMessage));
	ZeroMemory(szMessage2, sizeof(szMessage2));

	switch (iRet)
	{
	case SAM_PON:
		StringCchCopy(szMessage, nLen, _T("SAM already opened.\r\n"));
		break;
	case SAM_NOT_DEVICE:
		StringCchCopy(szMessage, nLen, _T("SAM driver error.\r\n"));
		break;
	case SAM_ERROR_INVALID_ACCESS:
		StringCchCopy(szMessage, nLen, _T("Device exclusive error.\r\n"));
		break;
	case SAM_ERROR_MODULE:
		StringCchCopy(szMessage, nLen, _T("SAM module not responce error.\r\n"));
		break;
	case SAM_POF:
		StringCchCopy(szMessage, nLen, _T("SAM not open error.\r\n"));
		break;
	case SAM_PRM:
		StringCchCopy(szMessage, nLen, _T("SAM parameter error.\r\n"));
		break;
	case SAM_ERROR_TIMEOUT:
		StringCchCopy(szMessage, nLen, _T("Timeout error.\r\n"));
		break;
	case SAM_NOT_ACTIVATION:
		StringCchCopy(szMessage, nLen, _T("Card don't start error.\r\n"));
		break;
	case SAM_ERROR_SUSPEND:
		StringCchCopy(szMessage, nLen, _T("Suspend error.\r\n"));
		break;
	case SAM_ERROR_RESPONSE:
		StringCchCopy(szMessage, nLen, _T("Card response error.\r\n"));
		break;
	case SAM_ERROR_NOCARD:
		StringCchCopy(szMessage, nLen, _T("Card does not exist.\r\n"));
		break;
	default:
		StringCchCopy(szMessage, nLen, _T("Other error.\r\n"));
		break;
	}
	wsprintf(szMessage2, _T("%s : %s\r\n"), szFunction, szMessage);
	OutputString(hWnd, szMessage2);

	SysSetBuzzerVolume(B_WARNING, BUZZERVOLUME_MAX);
	SysPlayBuzzer(B_WARNING, B_USERDEF, B_USERDEF);

	return TRUE;
}

BOOL OutputString(HWND hWnd, TCHAR *szString)
{
	int nLen = 0;
	int nLines = 0;
	nLen = GetWindowTextLength(hWnd);
	SendMessage(hWnd, EM_SETSEL, (WPARAM)nLen, (LPARAM)nLen);
	SendMessage(hWnd, EM_REPLACESEL, 0, (LPARAM)szString);
	nLines = SendMessage(hWnd, EM_GETLINECOUNT, 0, 0);
	SendMessage(hWnd, EM_LINESCROLL, 0, nLines);
	return TRUE;
}

BOOL DispHex(HWND hWnd, BYTE *pbyData, DWORD dwLen)
{
	int i = 0;
	ZeroMemory(szMessage, sizeof(szMessage));
	for (i=0 ; i<(int)dwLen ; i++)
	{
		wsprintf(&szMessage[i*3], _T("%02X "), pbyData[i]);
	}
	SetWindowText(hWnd, szMessage);
	return TRUE;
}
