/***************************************************************************
 * <Description>
 * Form_BUZZER.cs (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * System Library Demo program
 * -------------------------------------------------------------------------
 * <Language>
 * C#.NET
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * DT-5300 WM / IT-800 WM model
 * -------------------------------------------------------------------------
 * Copyright(C)2010 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2010.02.16      CASIO       000000      Original      
 * 
 ***************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Calib;
using CalibApi = Calib.SystemLibNet.Api;
using CalibDef = Calib.SystemLibNet.Def;

namespace SystemLibSampleCSWM
{
    public partial class Form_BUZZER : Form
    {

        // structure definition
        private struct tagBUZZERSETTING
        {
            public bool bMute;
            public int dwVol;
            tagBUZZERSETTING(
                bool bMute,
                int dwVol)
            {
                this.bMute = bMute;
                this.dwVol = dwVol;
            }
        }

        // MAX number
        private static int TYPE_MAXNUM = 6;

        // Type List
        private int[] dwTypeList = new int[TYPE_MAXNUM];

        // setting before change
        private tagBUZZERSETTING[] OldSetting = new tagBUZZERSETTING[TYPE_MAXNUM];

        // setting after change
        private tagBUZZERSETTING[] NewSetting = new tagBUZZERSETTING[TYPE_MAXNUM]; 


        public Form_BUZZER()
        {
            InitializeComponent();

            // buzzer type combo box initialize
            comboBox_Type.Items.Add("Click");
            dwTypeList[0] = CalibDef.B_CLICK;

            comboBox_Type.Items.Add("Alarm");
            dwTypeList[1] = CalibDef.B_ALARM;

            comboBox_Type.Items.Add("Warning");
            dwTypeList[2] = CalibDef.B_WARNING;

            comboBox_Type.Items.Add("Scanner");
            dwTypeList[3] = CalibDef.B_SCANEND;

            comboBox_Type.Items.Add("Tap");
            dwTypeList[4] = CalibDef.B_TAP;

            comboBox_Type.Items.Add("All Buzzer");
            dwTypeList[5] = CalibDef.B_ALL;


            // A present setting is preserved. 
            GetBuzzerSetting();

            comboBox_Type.SelectedIndex = 0;

        }

        private void button_Play_Click(object sender, EventArgs e)
        {
            int nSelType, dwRet;

            nSelType = comboBox_Type.SelectedIndex;

            //play
            dwRet = CalibApi.SysPlayBuzzer(
                        dwTypeList[nSelType], 0, 0);
            CommonFunc.ShowErrorMessage("SysPlayBuzzer", dwRet);

        }

        private void button_Stop_Click(object sender, EventArgs e)
        {
            int dwRet;

            //stop
            dwRet = CalibApi.SysStopBuzzer();
            CommonFunc.ShowErrorMessage("SysStopBuzzer", dwRet);
        }

        private void comboBox_Type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nNewSel;

            nNewSel = comboBox_Type.SelectedIndex;

            // change mute setting
            if (NewSetting[nNewSel].bMute == true)
            {
                checkBox_Mute.Checked = true;
            }
            else
            {
                checkBox_Mute.Checked = false;
            }

            // change volume setting
            comboBox_Volume.SelectedIndex = NewSetting[nNewSel].dwVol;


            // judge selected buzzer type is max sound or not
            if (dwTypeList[nNewSel] == CalibDef.B_ALL)
            {
                comboBox_Volume.Enabled = false;
                button_Play.Enabled = false;
            }
            else
            {
                comboBox_Volume.Enabled = true;
                button_Play.Enabled = true;
            }
        }

        private void checkBox_Mute_CheckStateChanged(object sender, EventArgs e)
        {
            Int32 dwRet;
            int nSel;

            nSel = comboBox_Type.SelectedIndex;

            NewSetting[nSel].bMute = checkBox_Mute.Checked;

            // mute setting set
            dwRet = CalibApi.SysSetBuzzerMute(dwTypeList[nSel], NewSetting[nSel].bMute);
            CommonFunc.ShowErrorMessage("SysSetBuzzerMute", dwRet);
        }

        private void comboBox_Volume_SelectedIndexChanged(object sender, EventArgs e)
        {
            Int32 dwRet;
            int dwVol;
            int nSel;

            nSel = comboBox_Type.SelectedIndex;

            NewSetting[nSel].dwVol = comboBox_Volume.SelectedIndex;

            // change Volume setting
            if (NewSetting[nSel].dwVol == 0) dwVol = CalibDef.BUZZERVOLUME_MIN;
            else if (NewSetting[nSel].dwVol == 1) dwVol = CalibDef.BUZZERVOLUME_MID;
            else dwVol = CalibDef.BUZZERVOLUME_MAX;
            dwRet = CalibApi.SysSetBuzzerVolume(dwTypeList[nSel], dwVol);
            CommonFunc.ShowErrorMessage("SysSetBuzzerVolume", dwRet);
        }

        // buzzer setting get
        private void GetBuzzerSetting()
        {
            Int32 dwMute;
            int dwVol;
            int i;

            for (i = 0; i < TYPE_MAXNUM; i++)
            {
                // mute setting get
                dwMute = CalibApi.SysGetBuzzerMute(dwTypeList[i]);
                CommonFunc.ShowErrorMessage("SysGetBuzzerMute", dwMute);
                if (dwMute != 0)
                {
                    NewSetting[i].bMute = OldSetting[i].bMute = true;
                }
                else
                {
                    NewSetting[i].bMute = OldSetting[i].bMute = false;
                }

                // volume get
                dwVol = CalibApi.SysGetBuzzerVolume(dwTypeList[i]);
                CommonFunc.ShowErrorMessage("SysGetBuzzerVolume", dwVol);

                if (dwVol == CalibDef.BUZZERVOLUME_MIN)
                {
                    NewSetting[i].dwVol = OldSetting[i].dwVol = 0;
                }
                else if (dwVol == CalibDef.BUZZERVOLUME_MID)
                {
                    NewSetting[i].dwVol = OldSetting[i].dwVol = 1;
                }
                else
                {
                    NewSetting[i].dwVol = OldSetting[i].dwVol = 2;
                }
            }

            return;
        }


        private void ResetSetting(object sender, System.EventArgs e)
        {
            int dwRet;
            int dwVol;
            int i;

            // return to before setting
            for (i = 0; i < TYPE_MAXNUM; i++)
            {
                // mute setting change
                dwRet = CalibApi.SysSetBuzzerMute(dwTypeList[i], OldSetting[i].bMute);
                CommonFunc.ShowErrorMessage("SysSetBuzzerMute", dwRet);

                // change Volume setting
                if (OldSetting[i].dwVol == 0) dwVol = CalibDef.BUZZERVOLUME_MIN;
                else if (OldSetting[i].dwVol == 1) dwVol = CalibDef.BUZZERVOLUME_MID;
                else dwVol = CalibDef.BUZZERVOLUME_MAX;
                dwRet = CalibApi.SysSetBuzzerVolume(dwTypeList[i], dwVol);
                CommonFunc.ShowErrorMessage("SysSetBuzzerVolume", dwRet);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.Close();
        }


    }
}