/***************************************************************************
 * <Description>
 * Form_VIBRATOR.cs (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * System Library Demo program
 * -------------------------------------------------------------------------
 * <Language>
 * C#.NET
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * DT-5300 WM / IT-800 WM model
 * -------------------------------------------------------------------------
 * Copyright(C)2010 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2010.02.16      CASIO       000000      Original      
 * 
 ***************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Calib;
using CalibApi = Calib.SystemLibNet.Api;
using CalibDef = Calib.SystemLibNet.Def;

namespace SystemLibSampleCSWM
{
    public partial class Form_VIBRATOR : Form
    {
        private static int dwTypeNum = 6;
        private int[] dwTypeList = new int[dwTypeNum];
        private bool[] OldSetting = new bool[dwTypeNum];

        public Form_VIBRATOR()
        {
            InitializeComponent();

            // vibration type combobox initialize
            comboBox_Type.Items.Add("Alarm");
            dwTypeList[0] = CalibDef.B_ALARM;

            comboBox_Type.Items.Add("Warning");
            dwTypeList[1] = CalibDef.B_WARNING;

            comboBox_Type.Items.Add("Scanner");
            dwTypeList[2] = CalibDef.B_SCANEND;

            comboBox_Type.Items.Add("Wireless read");
            dwTypeList[3] = CalibDef.B_WIREREAD;

            comboBox_Type.Items.Add("User Define");
            dwTypeList[4] = CalibDef.B_USERDEF;

            comboBox_Type.Items.Add("All Vibrator");
            dwTypeList[5] = CalibDef.B_ALL;

            // get vibration setting
            GetVibratorSetting();

            comboBox_Type.SelectedIndex = 0;
        }

        private void button_Play_Click(object sender, EventArgs e)
        {
            int dwRet;
            int nSelType;
            int dwNum = 0;
            int dwOnTime = 0;
            int dwOffTime = 0;

            nSelType = comboBox_Type.SelectedIndex;
            if (dwTypeList[nSelType] == CalibDef.B_USERDEF)
            {
                // vibration count
                dwNum = Convert.ToInt32(textBox_Num.Text);

                // vibration on time
                dwOnTime = Convert.ToInt32(textBox_OnTime.Text);

                // vibration off time
                dwOffTime = Convert.ToInt32(textBox_OffTime.Text);
            }

            // Play
            dwRet = CalibApi.SysPlayVibrator(
                dwTypeList[nSelType], dwNum, dwOnTime, dwOffTime);
            CommonFunc.ShowErrorMessage("SysPlayVibrator", dwRet);
        }

        private void button_Stop_Click(object sender, EventArgs e)
        {
            // Stop
            CalibApi.SysStopVibrator();
        }

        private void GetVibratorSetting()
        {
            int dwMute;
            int i;


            for (i = 0; i < dwTypeNum; i++)
            {
                // get mute setting
                dwMute = CalibApi.SysGetVibratorMute(dwTypeList[i]);
                CommonFunc.ShowErrorMessage("SysGetVibratorMute", dwMute);
                if (dwMute != 0)
                {
                    OldSetting[i] = true;
                }
                else
                {
                    OldSetting[i] = false;
                }
            }
        }

        private void comboBox_Type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int nSelType = comboBox_Type.SelectedIndex;


            // get mute setting
            if (CalibApi.SysGetVibratorMute(dwTypeList[nSelType]) != 0)
            {
                checkBox_Mute.Checked = true;
            }
            else
            {
                checkBox_Mute.Checked = false;
            }

            // judge selected vibration type is user setting or not
            if (dwTypeList[nSelType] == CalibDef.B_USERDEF)
            {
                // vibration count
                textBox_Num.Text = "2";
                textBox_Num.Enabled = true;

                // vibration on time
                textBox_OnTime.Text = "100";
                textBox_OnTime.Enabled = true;

                // vibration off time
                textBox_OffTime.Text = "100";
                textBox_OffTime.Enabled = true;
            }
            else
            {
                // disable vibration count
                textBox_Num.Text = "";
                textBox_Num.Enabled = false;

                // disable vibration on time
                textBox_OnTime.Text = "";
                textBox_OnTime.Enabled = false;

                // disable vibration off time
                textBox_OffTime.Text = "";
                textBox_OffTime.Enabled = false;
            }

            // judge selected vibration type is full vibration or not
            if (dwTypeList[nSelType] == CalibDef.B_ALL)
            {
                button_Play.Enabled = false;
            }
            else
            {
                button_Play.Enabled = true;
            }
        }

        private void checkBox_Mute_CheckStateChanged(object sender, EventArgs e)
        {
            int dwRet;
            int nSelType = comboBox_Type.SelectedIndex;


            // change mute setting
            dwRet = CalibApi.SysSetVibratorMute(dwTypeList[nSelType], checkBox_Mute.Checked);
            CommonFunc.ShowErrorMessage("SysSetVibratorMute", dwRet);		
        }

        private void ResetSetting(object sender, System.EventArgs e)
        {
            Int32 dwRet;
            int i;

            // return before setting
            for (i = 0; i < dwTypeNum; i++)
            {
                // change vibration setting
                dwRet = CalibApi.SysSetVibratorMute(dwTypeList[i], OldSetting[i]);
                CommonFunc.ShowErrorMessage("SysGetVibratorMute", dwRet);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.Close();
        }


    }
}