'***************************************************************************
'* <Description>
'* Form_Buzzer.vb (Casio Computer Co., Ltd.)
'* -------------------------------------------------------------------------
'* <Function outline>
'* System Library Demo program
'* -------------------------------------------------------------------------
'* <Language>
'* VB.NET
'* -------------------------------------------------------------------------
'* <Develop Environment>
'* VS 2005
'* -------------------------------------------------------------------------
'* <Target>
'* DT-5300 WM / IT-800 WM / IT-300 model
'* -------------------------------------------------------------------------
'* Copyright(C)2010 CASIO COMPUTER CO.,LTD. All rights reserved.
'* -------------------------------------------------------------------------
'* <History>
'* Version  Date            Company     Keyword     Comment
'* 1.0.0    2010.02.16      CASIO       000000      Original      
'* 
'***************************************************************************/
Imports Calib.SystemLibNet.Def
Imports Calib.SystemLibNet.Api

Public Class Form_Buzzer
    ' structure definition
    Private Structure tagBUZZERSETTING
        Public bMute As Boolean
        Public dwVol As Int32
    End Structure

    'MAX number
    Private Const TYPE_MAXNUM As Int32 = 6

    'Type List
    Private dwTypeList(TYPE_MAXNUM) As Int32

    ' setting before change
    Private OldSetting(TYPE_MAXNUM) As tagBUZZERSETTING

    ' setting after change
    Private NewSetting(TYPE_MAXNUM) As tagBUZZERSETTING



    Private Sub button_Play_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles button_Play.Click
        Dim nSelType As Int32, dwRet As Int32

        nSelType = comboBox_Type.SelectedIndex

        ' Play
        dwRet = SysPlayBuzzer(dwTypeList(nSelType), 0, 0)
        CommonFunc.ShowErrorMessage("SysPlayBuzzer", dwRet)
    End Sub

    Private Sub button_Stop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles button_Stop.Click
        Dim dwRet As Int32

        ' Stop
        dwRet = SysStopBuzzer()
        CommonFunc.ShowErrorMessage("SysStopBuzzer", dwRet)
    End Sub

    Private Sub Form_Buzzer_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' buzzer type combo box initialize
        comboBox_Type.Items.Add("Click")
        dwTypeList(0) = B_CLICK

        comboBox_Type.Items.Add("Alarm")
        dwTypeList(1) = B_ALARM

        comboBox_Type.Items.Add("Warning")
        dwTypeList(2) = B_WARNING

        comboBox_Type.Items.Add("Scanner")
        dwTypeList(3) = B_SCANEND

        comboBox_Type.Items.Add("Tap")
        dwTypeList(4) = B_TAP

        comboBox_Type.Items.Add("All Buzzer")
        dwTypeList(5) = B_ALL

        ' buzzer setting get
        GetBuzzerSetting()

        comboBox_Type.SelectedIndex = 0
    End Sub

    Private Sub GetBuzzerSetting()
        Dim dwMute As Int32
        Dim dwVol As Int32

        Dim i As Int32

        For i = 0 To TYPE_MAXNUM
            ' mute setting get
            dwMute = SysGetBuzzerMute(dwTypeList(i))
            ShowErrorMessage("SysGetBuzzerMute", dwMute)
            If (dwMute <> 0) Then
                NewSetting(i).bMute = True
                OldSetting(i).bMute = True
            Else
                NewSetting(i).bMute = False
                OldSetting(i).bMute = False
            End If

            ' volume setting get
            dwVol = SysGetBuzzerVolume(dwTypeList(i))
            ShowErrorMessage("SysGetBuzzerVolume", dwVol)

            If (dwVol = BUZZERVOLUME_MIN) Then
                NewSetting(i).dwVol = 0
                OldSetting(i).dwVol = 0
            ElseIf (dwVol = BUZZERVOLUME_MID) Then
                NewSetting(i).dwVol = 1
                OldSetting(i).dwVol = 1
            Else
                NewSetting(i).dwVol = 2
                OldSetting(i).dwVol = 2
            End If
        Next i

    End Sub

    Private Sub comboBox_Type_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles comboBox_Type.SelectedIndexChanged
        Dim nNewSel As Int32

        nNewSel = comboBox_Type.SelectedIndex

        ' change mute setting
        If (NewSetting(nNewSel).bMute = True) Then
            checkBox_Mute.Checked = True
        Else
            checkBox_Mute.Checked = False
        End If

        ' change sound setting
        comboBox_Volume.SelectedIndex = NewSetting(nNewSel).dwVol


        ' judge selected buzzer type is max sound or not
        If (dwTypeList(nNewSel) = B_ALL) Then
            comboBox_Volume.Enabled = False
            button_Play.Enabled = False
        Else
            comboBox_Volume.Enabled = True
            button_Play.Enabled = True
        End If
    End Sub

    Private Sub checkBox_Mute_CheckStateChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles checkBox_Mute.CheckStateChanged
        Dim dwRet As Int32
        Dim nSel As Int32

        nSel = comboBox_Type.SelectedIndex

        NewSetting(nSel).bMute = checkBox_Mute.Checked

        ' change mute setting
        dwRet = SysSetBuzzerMute(dwTypeList(nSel), NewSetting(nSel).bMute)
        CommonFunc.ShowErrorMessage("SysSetBuzzerMute", dwRet)
    End Sub

    Private Sub comboBox_Volume_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles comboBox_Volume.SelectedIndexChanged
        Dim dwRet As Int32
        Dim dwVol As Int32
        Dim nSel As Int32

        nSel = comboBox_Type.SelectedIndex

        NewSetting(nSel).dwVol = comboBox_Volume.SelectedIndex

        ' change sound setting
        If (NewSetting(nSel).dwVol = 0) Then
            dwVol = BUZZERVOLUME_MIN
        ElseIf (NewSetting(nSel).dwVol = 1) Then
            dwVol = BUZZERVOLUME_MID
        Else
            dwVol = BUZZERVOLUME_MAX
        End If

        dwRet = SysSetBuzzerVolume(dwTypeList(nSel), dwVol)
        ShowErrorMessage("SysSetBuzzerVolume", dwRet)
    End Sub


    Private Sub Form_Buzzer_Closed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        Dim dwRet As Int32
        Dim dwVol As Int32
        Dim i As Int32

        ' return to before setting
        For i = 0 To TYPE_MAXNUM

            ' change mute setting
            dwRet = SysSetBuzzerMute(dwTypeList(i), OldSetting(i).bMute)
            ShowErrorMessage("SysSetBuzzerMute", dwRet)

            ' change sound setting
            If (OldSetting(i).dwVol = 0) Then
                dwVol = BUZZERVOLUME_MIN
            ElseIf (OldSetting(i).dwVol = 1) Then
                dwVol = BUZZERVOLUME_MID
            Else
                dwVol = BUZZERVOLUME_MAX
            End If
            dwRet = SysSetBuzzerVolume(dwTypeList(i), dwVol)
            ShowErrorMessage("SysSetBuzzerVolume", dwRet)
        Next
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Me.Close()

    End Sub
End Class