/***************************************************************************
 * <Description>
 * Form1.cs (Casio Computer Co., Ltd.)
 * -------------------------------------------------------------------------
 * <Function outline>
 * WANGPRS communication Demo program
 * -------------------------------------------------------------------------
 * <Language>
 * C#.NET
 * -------------------------------------------------------------------------
 * <Develop Environment>
 * VS 2005
 * -------------------------------------------------------------------------
 * <Target>
 * DT-5300 CE  
 * -------------------------------------------------------------------------
 * Copyright(C)2009 CASIO COMPUTER CO.,LTD. All rights reserved.
 * -------------------------------------------------------------------------
 * <History>
 * Version  Date            Company     Keyword     Comment
 * 1.0.0    2009.10.14      CASIO       000000      Original      
 * 1.0.1    2012.05.28      CASIO       000001      Emulator error catch    
 * 
 ***************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Calib;

namespace WANGPRSLibSampleCECS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        /*! GLOBAL VARIABLEs  */
        static IntPtr g_hRasConn = IntPtr.Zero;             //!< RAS handle
        private const string g_szPIN_CODE = "1111";			//!< PIN code for SIM
        private const string g_RasEntryName = "WANG_GPRS";  //!< RAS entry name to make (UNICODE)
        private const string g_GPRS_APN = "anritsu.ne.jp";	//!< APN (Network Provider)
        private const string g_DefaultTEL = "*99***1#";     //!< Default TEL number

        private const Int32 ERROR_ALREADY_EXISTS = 183;
        private const Int32 ERROR_INVALID_NAME = 123;
        private const Int32 ERROR_SUCCESS = 0;

        //
        //! If your network provider requests to specify UserName and Password, please set these.
        //private const string g_RasUserName = "gprs";		//!< Username for network provider
        //private const string g_RasPassword = "gprs";		//!< Password for network provider

        [DllImport("coredll.dll")]
        private static extern int RasValidateEntryName(IntPtr lpszPhonebook, string lpszEntry);

        [DllImport("coredll.dll")]
        private static extern int RasDeleteEntry(IntPtr lpszPhonebook, string lpszEntry);

        private void button1_Click(object sender, EventArgs e)
        {   // COMMUNICATION WITH GPRS
            //
            if (turnOn_WANmodule() == false)
            {		                            // Power on GPRS module 
                goto error;						//
            }									//

            if (unlock_PINLOCK() == false)
            {		                            // Unlock SIM Pin
                goto error;						//
            }									//

            if (create_RASENTRY() == false)
            {		                            // Make RAS entry
                goto error;						//
            }									//

            if (connect_GPRS() == false)
            {			                        // RAS connection
                goto error;						//
            }									//

            if (notify_message() == false)
            {		                            // Display a message
                goto error;						//
            }									//

            if (disconnect_GPRS() == false)
            {		                            // Disconnection
                goto error;						//
            }									//

error:			  						        //

            turnOff_WANmodule();				// Power off GPRS module
            //
            //
            // Exit

        }

        /*! Power On GPRS module  */
        private bool turnOn_WANmodule()
        {
            bool bRet = false;
            Int32 dwRet = new Int32();

            AppendText("INFO:turnOn_WANmodule Enter.");

            //! To turn on the GPRS module, use this function.
            if ((dwRet = WANGPRSLibNet.Api.WanSetPowerStatus(WANGPRSLibNet.Def.PowerStatus.WAN_MODULE_POWER_ON)) != WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
            {
                AppendText("ERR:WANSetPowerStatus(ON)");
                AppendText(String.Format("   err={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet));
                bRet = false;
            }
            else
            {
                AppendText("INFO:WANSetPowerStatus(ON) Success.");
                bRet = true;
            }

            AppendText(String.Format("INFO:turnOn_WANmodule Leave({0}).", bRet.ToString()));
            return bRet;
        }

        /*! If your SIM card is locked by PIN code, you must unlock it. 
	        If you don't, you cannot communicate.
        */
        private bool unlock_PINLOCK()
        {
            bool bRet = false;
            Int32 dwRet = new Int32();

            //Functions
            IntPtr hSIM = IntPtr.Zero;
            WANGPRSLibNet.Def.SimLockStatus LockState = new WANGPRSLibNet.Def.SimLockStatus();

            AppendText("INFO:unlock_PINLOCK Enter.");

            //! Before operating SIM related functions, you should get a handle. 
            if ((dwRet = WANGPRSLibNet.Api.WanSimInitialize(out hSIM)) != WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
            {
                AppendText("ERR:WANSIMInitialize");
                AppendText(String.Format("    error={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet));
                bRet = false;
                goto error;
            }


            //! Checking the SIM status (Locked or not)
            if ((dwRet = WANGPRSLibNet.Api.WanSimGetPhoneLockedState(hSIM, out LockState)) != WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
            {
                AppendText("ERR:WANSIMGetPhoneLockedState");
                AppendText(String.Format("    error={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet));
                bRet = false;
                goto error;
            }

            /*! If the status is WAN_SIM_LOCKSTATE_PIN, unlocking the PIN lock  */
            if (LockState == WANGPRSLibNet.Def.SimLockStatus.WAN_SIM_LOCKSTATE_PIN)
            {
                /*! Set right PIN code to g_szPIN_CODE and call WANSIMUnLockPhone. */
                if ((dwRet = WANGPRSLibNet.Api.WanSimUnlockPhone(hSIM, g_szPIN_CODE, "")) != WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
                {
                    /*! Faild to unlock. You should check if the PIN code is right.  */
                    AppendText("ERR:WANSIMUnLockPhone");
                    AppendText(String.Format("    error={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet));
                    bRet = false;
                }
                else
                {
                    //! Unlocked
                    bRet = true;
                }
            }
            /*!  If the status is WAN_SIM_LOCKSTATE_READY, already unlocked or no SIM sucurity.  */
            else if (LockState == WANGPRSLibNet.Def.SimLockStatus.WAN_SIM_LOCKSTATE_READY)
            {

                return true;

            }
            /*! In other case, you can't use your SIM with DT-X30. Please access your SIM supplier.  */
            else
            {
                //!ERROR -> other locks..
                AppendText("ERR:Security Error.");
                bRet = false;
                goto error;
            }

        error:

            /*!  You should Deinit the Handle after using SIM function.  */
            if (hSIM != IntPtr.Zero)
            {
                if ((dwRet = WANGPRSLibNet.Api.WanSimDeinitialize(hSIM)) != WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
                {
                    AppendText("ERR:WANSIMDeInitialize");
                    AppendText(String.Format("    error={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet));

                }
            }

            AppendText(String.Format("INFO:unlock_PINLOCK Leave({0}).", bRet.ToString()));
            return bRet;
        }

        /*! Making RAS entry
            @retval true	Success
                    false	Fail
        */
        private bool create_RASENTRY()
        {
            bool bRet = false;
            Int32 dwRet = new Int32();

            AppendText("INFO:create_RASENTRY Enter.");

            // check if the named RAS entry already exist or not
            AppendText("INFO:RAS Entry Checking ");
            if ((dwRet = RasValidateEntryName(IntPtr.Zero, g_RasEntryName)) != 0)
            {
                if (dwRet == ERROR_ALREADY_EXISTS)
                {
                    /*! If it already exists, delete it. (This is only for sample. 
                      If you use existing entry, you can use it.)
                    */
                    AppendText("INFO:RAS Entry already exist!.");
                    AppendText("     Trying to delete this entry..");

                    // if Windows Mobile
                    //XmlDocument configDoc = new XmlDocument();
                    //configDoc.LoadXml(
                    //"<wap-provisioningdoc>" +
                    //"<characteristic type=\"CM_PPPEntries\">" +
                    //   "<nocharacteristic type=\"" + g_RasEntryName + "\" />" +
                    //"</characteristic>" +
                    //"</wap-provisioningdoc>");

                    //ConfigurationManager.ProcessConfiguration(configDoc, false);
                    //AppendText("INFO:RAS Entry delete OK");
                    // end Windows Mobile

                    // if Windows CE
                    if ((dwRet = RasDeleteEntry(IntPtr.Zero, g_RasEntryName)) != 0)
                    {
                        AppendText("ERR:RAS Entry delete Fail");
                        AppendText(String.Format("    (RasDeleteEntry)err={0}..", dwRet));
                        goto error;
                    }
                    //! success to delete existing entry
                    else
                    {
                        AppendText("INFO:RAS Entry delete OK");
                    }
                    // end Windows CE
                }
                //! your specified entry name is not accepted by MS RAS functions
                else if (dwRet == ERROR_INVALID_NAME)
                {
                    AppendText("ERR:RAS Entry Invalid");
                    AppendText("   it has been unexpected..");
                    goto error;
                }
            }
            else
            {
                //! your specified entry name is not existing
                AppendText("INFO:NO EXISTING RAS Entry.");
            }



            //! make RAS entry
            if ((dwRet = WANGPRSLibNet.Api.WanSetRasSetting(g_RasEntryName, g_DefaultTEL, g_GPRS_APN)) != WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
            {
                AppendText("ERR:RAS Entry can't create!");
                AppendText(String.Format("    (RasDeleteEntry)err={0}..", dwRet));
                goto error;
            }

            //SUCCESS
            bRet = true;

        error:

            AppendText(String.Format("INFO:create_RASENTRY Leave({0}).", bRet.ToString()));
            return bRet;
        }

        /*! RAS Connection */
        private bool connect_GPRS()
        {
            bool bRet = false;
            Int32 dwRet = new Int32();
            Int32 dwRASResult = new Int32();

            AppendText("INFO:connect_GPRS Enter.");

            //! set the member of RASDIALPARAMS
            WANGPRSLibNet.RasDialParams rasDialParam = new WANGPRSLibNet.RasDialParams(g_RasEntryName);

            //! If your network provider requests to specify UserName and Password, please set these.
            //!< User name for RAS connection
            //rasDialParam.UserName = g_RasUserName;
            //!< Password for RAS connection
            //rasDialParam.Password = g_RasPassword;

            //! Do RAS Conncetion
            AppendText("INFO:WANRASConnect ");
            AppendText("     Connecting");

            if ((dwRet = WANGPRSLibNet.Api.WanRasConnect(out dwRASResult, rasDialParam, WANGPRSLibNet.Def.RasDialNotifierType.NoNotifier,
                                                         IntPtr.Zero, out g_hRasConn)) == WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
            {
                if (dwRASResult != ERROR_SUCCESS)
                {
                    //! RasDial() fails. WANRASConnect() calls MS's RasDial().
                    AppendText("ERR:WANRASConnect Fail");
                    AppendText(String.Format("   RASRESULT={0}", dwRASResult));
                    bRet = false;
                    goto error;
                }
                else
                {
                    //! Success
                    AppendText("INFO:WANRASConnect Connected");
                }
            }
            else
            {
                //! WANRASConnect fails. Please check the return code to find the reason.
                AppendText("ERR:WANRASConnect ");
                AppendText(String.Format("    Fail WANerr={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet));
                bRet = false;
                goto error;
            }

            //! Success
            bRet = true;

        error:

            AppendText(String.Format("INFO:connect_GPRS Leave({0}).", bRet.ToString()));
            return bRet;
        }

        private bool notify_message()
        {
            bool bRet = true;

            if (MessageBox.Show("Disconnect???", "INFO") != DialogResult.OK)
            {
                //MessageBox failure...
                bRet = false;
            }

            return bRet;
        }

        /*! Disconnect the RAS connection */
        private bool disconnect_GPRS()
        {
            Int32 dwRet = new Int32();
            bool bRet;
            Int32 dwRASResult = new Int32();

            AppendText("INFO:disconnect_GPRS Enter.");

            try //000001
            {
                if ((dwRet = WANGPRSLibNet.Api.WanRasDisconnect(out dwRASResult, g_hRasConn)) == WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
                {
                    if (dwRASResult != ERROR_SUCCESS)
                    {
                        //! RasHangUp() fails. WANRASDisConnect() calls MS's RasHangUp().
                        AppendText("ERR:WANRASDisConnect Fail ");
                        AppendText(String.Format("    RASRESULT={0}", dwRASResult));
                        bRet = false;
                    }
                    else
                    {
                        //! SUCCESS
                        g_hRasConn = IntPtr.Zero;
                        bRet = true;
                    }
                }
                else
                {
                    AppendText("ERR:WANRASDisConnect Fail ");
                    AppendText(String.Format("    WANerr={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet));
                    bRet = false;
                }
            }
            catch (System.Exception e) //000001
            {
                AppendText("ERR:WANRASDisConnect Fail ");
                AppendText(String.Format("    WANerr={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet));
                bRet = false;
            }


            AppendText(String.Format("INFO:disconnect_GPRS Leave({0}).", bRet.ToString()));
            return bRet;
        }

        /*! Power off the GPRS module */
        private bool turnOff_WANmodule()
        {
            bool bRet = true;
            Int32 dwRet = new Int32();

            AppendText("INFO:turnOff_WANmodule Enter.");

            //Power off the GPRS module
            if ((dwRet = WANGPRSLibNet.Api.WanSetPowerStatus(WANGPRSLibNet.Def.PowerStatus.WAN_MODULE_POWER_OFF)) != WANGPRSLibNet.Def.WAN_ERROR_SUCCESS)
            {
                AppendText("ERR:WANSetPowerStatus(OFF) ");
                AppendText(String.Format("    err={0}", dwRet));
                bRet = false;
                goto error;
            }
        error:
            AppendText(String.Format("INFO:turnOff_WANmodule Leave({0}).", bRet.ToString()));
            return bRet;
        }

        private void AppendText(String output)
        {
            String buff = textBox1.Text;
            textBox1.Text = buff + output + "\r\n";
            textBox1.Focus();
            textBox1.SelectionStart = textBox1.Text.Length;
            textBox1.ScrollToCaret();
        }

        static string WANGPRS_ERROR_IDtoString(Int32 returnCode)
        {
            string error;
            switch (returnCode)
            {
                case WANGPRSLibNet.Def.WAN_BUFFER_ERROR:
                    error = "BUFFER_ERROR 1";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_BADPARAM:
                    error = "BADPARAM";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_BUFFER_ERROR:
                    error = "BUFFER_ERROR 3";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_BUSY:
                    error = "BUSY";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_FAIL:
                    error = "FAIL";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_ILLEGAL_HANDLE:
                    error = "ILLEGAL_HANDLE";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_INCORRECT_PASSWORD:
                    error = "INCORRECT_PASSWORD";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_MODULE_NOT_READY:
                    error = "MODULE_NOT_READY";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_MORE_OPERATOR:
                    error = "MORE_OPERATOR";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_NETWORK_NOT_AVAILABLE:
                    error = "NETWORK_NOT_AVAILABLE";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_NO_OPERATOR:
                    error = "NO_OPERATOR";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_NOMAIL:
                    error = "NOMAIL";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_NOSIM:
                    error = "NOSIM";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_NOT_IMPLEMENT:
                    error = "NOT_IMPLEMENT";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_OPERATION_NOT_ALLOW:
                    error = "OPERATION_NOT_ALLOW";
                    break;
                case WANGPRSLibNet.Def.WAN_ERROR_SUCCESS:
                    error = "SUCCESS";
                    break;
                default:
                    error = "* unknown return value *";
                    break;
            }
            return error;
        }



    }
}