'***************************************************************************
'* <Description>
'* Form1.vb (Casio Computer Co., Ltd.)
'* -------------------------------------------------------------------------
'* <Function outline>
'* WANGPRS communication Demo program
'* -------------------------------------------------------------------------
'* <Language>
'* VB.NET
'* -------------------------------------------------------------------------
'* <Develop Environment>
'* VS 2005
'* -------------------------------------------------------------------------
'* <Target>
'* DT-5300 CE model
'* -------------------------------------------------------------------------
'* Copyright(C)2009 CASIO COMPUTER CO.,LTD. All rights reserved.
'* -------------------------------------------------------------------------
'* <History>
'* Version  Date            Company     Keyword     Comment
'* 1.0.0    2009.10.14      CASIO       000000      Original      
'* 1.0.1    2012.05.28      CASIO       000001      Emulator error catch    
'* 
'***************************************************************************/

Imports System.Windows.Forms

Public Class Form1
    ' GLOBAL VARIABLEs
    Dim g_hRasConn As IntPtr = New IntPtr()              '!< RAS handle
    Private Const g_szPIN_CODE As String = "1111"        '!< PIN code for SIM
    Private Const g_RasEntryName As String = "WANG_GPRS" '!< RAS entry name to make
    Private Const g_GPRS_APN As String = "anritsu.ne.jp" '!< APN (Network Provider)
    Private Const g_DefaultTEL As String = "*99***1#"    '!< Default TEL number

    '
    ' If your network provider requests to specify UserName and Password, please set these.
    ' Private Const g_RasUserName As String = "gprs";		//!< Username for network provider
    ' Private Const g_RasPassword As String = "gprs";		//!< Password for network provider

    Private Const ERROR_ALREADY_EXISTS As Int32 = 183
    Private Const ERROR_INVALID_NAME As Int32 = 123
    Private Const ERROR_SUCCESS As Int32 = 0

    Declare Function RasValidateEntryName Lib "coredll.dll" _
                        (ByVal lpszPhonebook As IntPtr, ByVal lpszEntryPassKey As String) As Integer

    Declare Function RasDeleteEntry Lib "coredll.dll" _
                        (ByVal lpszPhonebook As IntPtr, ByVal lpszEntryPassKey As String) As Integer


    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        ' COMMUNICATION WITH GPRS
        '
        If (turnOn_WANmodule() = False) Then    ' Power on GPRS module
            GoTo error_out
        End If

        If (unlock_PINLOCK() = False) Then      ' Unlock SIM Pin
            GoTo error_out
        End If

        If (create_RASENTRY() = False) Then     ' Make RAS entry
            GoTo error_out
        End If

        If (connect_GPRS() = False) Then        ' RAS connection
            GoTo error_out
        End If

        If (notify_message() = False) Then      ' Display a message
            GoTo error_out
        End If

        If (disconnect_GPRS() = False) Then     ' Disconnection
            GoTo error_out
        End If

error_out:
        turnOff_WANmodule()                 ' Power off GPRS module
        ' Exit
    End Sub

    ' Power On GPRS module
    Private Function turnOn_WANmodule() As Boolean
        Dim bRet As Boolean = False
        Dim dwRet As Int32


        AppendText("INFO:turnOn_WANmodule Enter.")

        ' To turn on the GPRS module, use this function.
        dwRet = Calib.WANGPRSLibNet.Api.WanSetPowerStatus(Calib.WANGPRSLibNet.Def.PowerStatus.WAN_MODULE_POWER_ON)
        If (dwRet <> Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
            AppendText("ERR:WANSetPowerStatus(ON)")
            AppendText(String.Format("   err={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet))
            bRet = False
        Else
            AppendText("INFO:WANSetPowerStatus(ON) Success.")
            bRet = True
        End If

        AppendText(String.Format("INFO:turnOn_WANmodule Leave({0}).", bRet.ToString()))

        Return bRet
    End Function

    ' If your SIM card is locked by PIN code, you must unlock it. 
    ' If you don't, you cannot communicate.
    Private Function unlock_PINLOCK() As Boolean
        Dim bRet As Boolean = False
        Dim dwRet As Int32

        'Functions
        Dim hSIM As New IntPtr
        Dim LockState As Calib.WANGPRSLibNet.Def.SimLockStatus

        AppendText("INFO:unlock_PINLOCK Enter.")

        ' Before operating SIM related functions, you should get a handle. 
        dwRet = Calib.WANGPRSLibNet.Api.WanSimInitialize(hSIM)
        If (dwRet <> Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
            AppendText("ERR:WANSIMInitialize")
            AppendText(String.Format("    error={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet))
            bRet = False
            GoTo error_out
        End If

        ' Checking the SIM status (Locked or not)
        dwRet = Calib.WANGPRSLibNet.Api.WanSimGetPhoneLockedState(hSIM, LockState)
        If (dwRet <> Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
            AppendText("ERR:WANSIMGetPhoneLockedState")
            AppendText(String.Format("    error={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet))
            bRet = False
            GoTo error_out
        End If

        ' If the status is WAN_SIM_LOCKSTATE_PIN, unlocking the PIN lock
        If (LockState = Calib.WANGPRSLibNet.Def.SimLockStatus.WAN_SIM_LOCKSTATE_PIN) Then
            ' Set right PIN code to g_szPIN_CODE and call WANSIMUnLockPhone.
            dwRet = Calib.WANGPRSLibNet.Api.WanSimUnlockPhone(hSIM, g_szPIN_CODE, "")
            If (dwRet <> Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
                ' Faild to unlock. You should check if the PIN code is right.
                AppendText("ERR:WANSIMUnLockPhone")
                AppendText(String.Format("    error={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet))
                bRet = False

            Else
                '! Unlocked
                bRet = True

            End If

            '  If the status is WAN_SIM_LOCKSTATE_READY, already unlocked or no SIM sucurity.
        ElseIf (LockState = Calib.WANGPRSLibNet.Def.SimLockStatus.WAN_SIM_LOCKSTATE_READY) Then
            Return True

            ' In other case, you can't use your SIM with DT-X30. Please access your SIM supplier.
        Else
            '!ERROR -> other locks..
            AppendText("ERR:Security Error.")
            bRet = False
            GoTo error_out

        End If

error_out:

        '  You should Deinit the Handle after using SIM function.
        If (hSIM <> IntPtr.Zero) Then
            dwRet = Calib.WANGPRSLibNet.Api.WanSimDeinitialize(hSIM)
            If (dwRet <> Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
                AppendText("ERR:WANSIMDeInitialize")
                AppendText(String.Format("    error={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet))
            End If
        End If

        AppendText(String.Format("INFO:unlock_PINLOCK Leave({0}).", bRet.ToString()))
        Return bRet
    End Function


    ' Making RAS entry
    '   @retval true	Success
    '           false	Fail
    Private Function create_RASENTRY() As Boolean
        Dim bRet As Boolean = False
        Dim dwRet As Int32

        AppendText("INFO:create_RASENTRY Enter.")

        ' check if the named RAS entry already exist or not
        AppendText("INFO:RAS Entry Checking ")
        dwRet = RasValidateEntryName(IntPtr.Zero, g_RasEntryName)
        If (dwRet <> 0) Then
            If (dwRet = ERROR_ALREADY_EXISTS) Then
                ' If it already exists, delete it. (This is only for sample. 
                ' If you use existing entry, you can use it.)
                AppendText("INFO:RAS Entry already exist!.")
                AppendText("     Trying to delete this entry..")

                ' If Windows Mobile
                'Dim configDoc As New System.Xml.XmlDocument()
                'configDoc.LoadXml( _
                '   "<wap-provisioningdoc>" & _
                '   "<characteristic type=""CM_PPPEntries"">" & _
                '      "<nocharacteristic type=""" + g_RasEntryName + """ />" & _
                '   "</characteristic>" & _
                '   "</wap-provisioningdoc>")

                'Microsoft.WindowsMobile.Configuration.ConfigurationManager.ProcessConfiguration(configDoc, False)
                'AppendText("INFO:RAS Entry delete OK")
                ' End Windows Mobile

                ' If Windows CE
                dwRet = RasDeleteEntry(IntPtr.Zero, g_RasEntryName)
                If (dwRet <> 0) Then
                    AppendText("ERR:RAS Entry delete Fail")
                    AppendText(String.Format("    (RasDeleteEntry)err={0}.", dwRet))
                    GoTo error_out
                    'success to delete existing entry
                Else
                    AppendText("INFO:RAS Entry delete OK")
                End If
                ' End Windows CE

                ' your specified entry name is not accepted by MS RAS functions
            ElseIf (dwRet = ERROR_INVALID_NAME) Then
                AppendText("ERR:RAS Entry Invalid")
                AppendText("   it has been unexpected..")
                GoTo error_out
            End If

        Else
            'your specified entry name is not existing
            AppendText("INFO:NO EXISTING RAS Entry.")

        End If

        ' make RAS entry
        dwRet = Calib.WANGPRSLibNet.Api.WanSetRasSetting(g_RasEntryName, g_DefaultTEL, g_GPRS_APN)
        If (dwRet <> Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
            AppendText("ERR:RAS Entry can't create!")
            AppendText(String.Format("    (RasDeleteEntry)err={0}..", dwRet))
            GoTo error_out
        End If

        ' SUCCESS
        bRet = True

error_out:

        AppendText(String.Format("INFO:create_RASENTRY Leave({0}).", bRet.ToString()))
        Return bRet
    End Function


    ' RAS Connection
    Private Function connect_GPRS() As Boolean
        Dim bRet As Boolean = False
        Dim dwRet As Int32
        Dim dwRASResult As Int32

        AppendText("INFO:connect_GPRS Enter.")

        ' set the member of RASDIALPARAMS
        Dim rasDialParam As New Calib.WANGPRSLibNet.RasDialParams(g_RasEntryName)

        ' If your network provider requests to specify UserName and Password, please set these.
        ' < User name for RAS connection
        ' rasDialParam.UserName = g_RasUserName
        ' < Password for RAS connection
        ' rasDialParam.Password = g_RasPassword

        ' Do RAS Conncetion
        AppendText("INFO:WANRASConnect ")
        AppendText("     Connecting")

        dwRet = Calib.WANGPRSLibNet.Api.WanRasConnect(dwRASResult, rasDialParam, Calib.WANGPRSLibNet.Def.RasDialNotifierType.NoNotifier, IntPtr.Zero, g_hRasConn)
        If (dwRet = Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
            If (dwRASResult <> ERROR_SUCCESS) Then
                ' RasDial() fails. WANRASConnect() calls MS's RasDial().
                AppendText("ERR:WANRASConnect Fail")
                AppendText(String.Format("   RASRESULT={0}", dwRASResult))
                bRet = False
                GoTo error_out

            Else
                ' Success
                AppendText("INFO:WANRASConnect Connected")

            End If

        Else
            ' WANRASConnect fails. Please check the return code to find the reason.
            AppendText("ERR:WANRASConnect ")
            AppendText(String.Format("    Fail WANerr={0}({1}) ", WANGPRS_ERROR_IDtoString(dwRet), dwRet))
            bRet = False
            GoTo error_out

        End If

        ' Success
        bRet = True

error_out:

        AppendText(String.Format("INFO:connect_GPRS Leave({0}).", bRet.ToString()))
        Return bRet
    End Function


    Private Function notify_message() As Boolean
        Dim bRet As Boolean = True

        If (MessageBox.Show("Disconnect???", "INFO") <> DialogResult.OK) Then
            ' MessageBox failure...
            bRet = False
        End If

        Return bRet
    End Function

    ' Disconnect the RAS connection
    Private Function disconnect_GPRS() As Boolean
        Dim bRet As Boolean = False
        Dim dwRet As Int32
        Dim dwRASResult As Int32

        AppendText("INFO:disconnect_GPRS Enter.")

        Try '000001
            dwRet = Calib.WANGPRSLibNet.Api.WanRasDisconnect(dwRASResult, g_hRasConn)
            If (dwRet = Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
                If (dwRASResult <> ERROR_SUCCESS) Then
                    ' RasHangUp() fails. WANRASDisConnect() calls MS's RasHangUp().
                    AppendText("ERR:WANRASDisConnect Fail ")
                    AppendText(String.Format("    RASRESULT={0}", dwRASResult))
                    bRet = False
                Else
                    ' SUCCESS
                    g_hRasConn = IntPtr.Zero
                    bRet = True

                End If
            Else
                AppendText("ERR:WANRASDisConnect Fail ")
                AppendText(String.Format("    WANerr={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet))
                bRet = False
            End If
        Catch ex As System.Exception '000001
            AppendText("ERR:WANRASDisConnect Fail ")
            AppendText(String.Format("    WANerr={0}({1})", WANGPRS_ERROR_IDtoString(dwRet), dwRet))
            bRet = False
        End Try

        AppendText(String.Format("INFO:disconnect_GPRS Leave({0}).", bRet.ToString()))
        Return bRet
    End Function


    ' Power off the GPRS module
    Private Function turnOff_WANmodule() As Boolean
        Dim bRet As Boolean = True
        Dim dwRet As Int32

        AppendText("INFO:turnOff_WANmodule Enter.")

        'Power off the GPRS module
        dwRet = Calib.WANGPRSLibNet.Api.WanSetPowerStatus(Calib.WANGPRSLibNet.Def.PowerStatus.WAN_MODULE_POWER_OFF)
        If (dwRet <> Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS) Then
            AppendText("ERR:WANSetPowerStatus(OFF) ")
            AppendText(String.Format("    err={0}", dwRet))
            bRet = False
            GoTo error_out
        End If

error_out:

        AppendText(String.Format("INFO:turnOff_WANmodule Leave({0}).", bRet.ToString()))
        Return bRet
    End Function


    Private Sub AppendText(ByVal output As String)
        Dim buff As String

        With TextBox1
            buff = .Text
            .Text = buff + output + vbCrLf
            .Focus()
            .SelectionStart = .Text.Length
            .ScrollToCaret()
        End With

    End Sub


    Private Function WANGPRS_ERROR_IDtoString(ByVal returnCode As Int32) As String
        Dim errorID As String

        Select Case (returnCode)
            Case Calib.WANGPRSLibNet.Def.WAN_BUFFER_ERROR
                errorID = "BUFFER_ERROR 1"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_BADPARAM
                errorID = "BADPARAM"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_BUFFER_ERROR
                errorID = "BUFFER_ERROR 3"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_BUSY
                errorID = "BUSY"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_FAIL
                errorID = "FAIL"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_ILLEGAL_HANDLE
                errorID = "ILLEGAL_HANDLE"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_INCORRECT_PASSWORD
                errorID = "INCORRECT_PASSWORD"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_MODULE_NOT_READY
                errorID = "MODULE_NOT_READY"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_MORE_OPERATOR
                errorID = "MORE_OPERATOR"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_NETWORK_NOT_AVAILABLE
                errorID = "NETWORK_NOT_AVAILABLE"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_NO_OPERATOR
                errorID = "NO_OPERATOR"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_NOMAIL
                errorID = "NOMAIL"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_NOSIM
                errorID = "NOSIM"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_NOT_IMPLEMENT
                errorID = "NOT_IMPLEMENT"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_OPERATION_NOT_ALLOW
                errorID = "OPERATION_NOT_ALLOW"

            Case Calib.WANGPRSLibNet.Def.WAN_ERROR_SUCCESS
                errorID = "SUCCESS"

            Case Else
                errorID = "* unknown return value *"
        End Select

        Return errorID
    End Function
End Class
